/*
 * Copyright (c) 2002-2003 Paul Vojta and the xdvik project team
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL ANY AUTHO OF THIS SOFTWARE BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef PRINT_INTERNAL_H
#define PRINT_INTERNAL_H

#include "dvi-init.h"
#include "print-dialog.h"

typedef enum { FMT_NONE, FMT_PS, FMT_PS2PDF, FMT_DVI, FMT_ISO_8859_1, FMT_UTF8 } output_format_t;

/* file IO help struct */
struct file_io_t {
    FILE *fp;
    char *fname;
};

/* collection of all file IOs used in printing/saving */
struct file_info_t {
    struct file_io_t dvi_in;
    struct file_io_t dvi_tmp;
    struct file_io_t dvi_out;
    struct file_io_t ps_out;
    struct file_io_t pdf_out;
    struct file_io_t txt_out;
};

/* stacks for communication with selection routines */
struct specials_stack_elem_t {
    char *content;		/* string content */
};

struct specials_stack_t {
    size_t stack_len;
    struct specials_stack_elem_t *items;
};


/* wrapper struct for all kinds of information about selected pages. */
struct select_pages_info_t {
    print_or_save_action_t act;	/* whether we're printing or saving */
    int from;			/* lower bound of page range to be selected */
    int to;			/* upper bound of page range to be selected */
    struct file_info_t *finfo;	/* additional file info pointer */
    /* callback function that says whether a page should be selected or not;
       will be passed a pointer to the current struct select_pages_info_t,
       and the current page */
    Boolean (*callback)(struct select_pages_info_t *info, int page);
    struct specials_stack_t warn_files;	/* collect warnings about included files */
    dvi_err_flag_t errflag;	/* collect all kinds of errors that can happen while copying */
};

extern void internal_print(struct select_pages_info_t *pinfo);
extern void internal_save(struct select_pages_info_t *pinfo,
			  output_format_t output_format);
#endif /* PRINT_INTERNAL_H */

