/*
 * Copyright (c) 2003 Xdvik project team
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef DVI_SEARCH_H
#define DVI_SEARCH_H

#include "xdvi-config.h"
#include "xdvi.h"
#include "search-dialog.h"
#include "dvisel.h"

/* A sorted dynamic array that maps text buffer (i.e. word)
 * positions to DVI file offsets.
 */
struct pos_info_t {
    size_t buffer_pos;	/* position in text buffer */
    struct bbox_t bbox;	/* bounding box of this word */
};

struct page_mapping_t {
    int pageno;
    int offset;
};

struct word_info_t {
    char *txt_buf;	 /* contains scanned text as whitespace separated strings */
    size_t txt_buf_size; /* size of above buffer */
    size_t curr_buf_idx;	 /* current position in buffer */
    struct bbox_t *bboxes;	/* dynamic array of position infos */
    size_t bboxes_size;	/* size of above array */
    size_t bboxes_idx;	 /* current position in buffer */
    const struct page_mapping_t *page_mapping;		/* list of page offsets, for bbox_pass */
    struct search_settings_t *settings;	/* pointer to search settings, for bbox_pass */
    Boolean bbox_pass;		/* whether we're scanning for bounding boxes of match */
};

extern void search_dvi(XtPointer settings);
extern void search_restart(XtPointer settings);
extern Boolean search_extract_text(output_format_t fmt, struct select_pages_info_t *pinfo);
extern Boolean search_have_match(int pageno);
extern int search_inside_bbox_match(int x, int y);
extern void search_draw_inverted_regions(void);
extern void search_draw_inverted_region(int x, int y);
extern void search_generate_expose(void);
extern void search_signal_page_changed(void);
extern void search_reset_info(void);
extern void search_erase_highlighting(void);

#endif /* DVI_SEARCH_H */

