/*
 * Copyright (c) 2002 xdvik development team
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef SPECIAL_H_
#define SPECIAL_H_

#include "xdvi-config.h"
#include "xdvi.h"

/*
 * If we're in the middle of a PSFIG special.
 */
extern Boolean psfig_begun;

extern Boolean scan_special(char *, void *data);

/*
 * Set if the -paper option overrides papersize specials.
 */
extern Boolean ignore_papersize_specials;

#if PS

extern struct psprocs {
    void (*toggle)	(int flag);
    void (*destroy)	(void);
    void (*interrupt)	(void);
    void (*endpage)	(void);
    void (*drawbegin)	(int, int, _Xconst char *);
    void (*drawraw)	(_Xconst char *);
    void (*drawfile)	(_Xconst char *, FILE *);
    void (*drawend)	(_Xconst char *);
    void (*beginheader)	(void);
    void (*endheader)	(void);
    void (*newdoc)	(void);
} psp, no_ps_procs;

extern void ps_clear_cache(void);
extern void ps_destroy(void);
extern void drawbegin_none(int, int, _Xconst char *);
extern void draw_bbox(void);
extern void display_bboxes(void);
extern void clear_bboxes(void);
extern void save_bbox(void);

extern Boolean have_raw_postscript;
#ifdef PS_GS
extern Boolean had_ps_specials;
#endif
#endif

extern void applicationDoSpecial(char *str, size_t len);
extern void geom_do_special(struct scan_info_t *, char *, double);
extern void color_special(_Xconst char *colorspec);

extern void init_prescan(void);

/*
  info on whether this DVI file contains papersize specials,
  and a resetting method.
*/
extern void reset_papersize_special(void);
extern Boolean have_papersize_special(void);

#if COLOR
extern void scan_color_eop(void);
struct rgb; /* forward declaration */
extern void set_fg_color(_Xconst struct rgb *);
#endif

#ifdef MAGICK
extern int bbox_angle;
extern Boolean bbox_valid;
extern unsigned int bbox_width;
extern unsigned int bbox_height;
extern int bbox_voffset;
#endif

#endif /* SPECIAL_H_ */
