/*========================================================================*\

Copyright (c) 1990-2002  Paul Vojta and others

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL PAUL
VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

NOTE: xdvi is based on prior work as noted in the modification history, below.

\*========================================================================*/

/*
 * DVI previewer for X.
 *
 * Eric Cooper, CMU, September 1985.
 *
 * Code derived from dvi-imagen.c.
 *
 * Modification history:
 * 1/1986	Modified for X.10	--Bob Scheifler, MIT LCS.
 * 7/1988	Modified for X.11	--Mark Eichin, MIT
 * 12/1988	Added 'R' option, toolkit, magnifying glass
 *					--Paul Vojta, UC Berkeley.
 * 2/1989	Added tpic support	--Jeffrey Lee, U of Toronto
 * 4/1989	Modified for System V	--Donald Richardson, Clarkson Univ.
 * 3/1990	Added VMS support	--Scott Allendorf, U of Iowa
 * 7/1990	Added reflection mode	--Michael Pak, Hebrew U of Jerusalem
 * 1/1992	Added greyscale code	--Till Brychcy, Techn. Univ. Muenchen
 *					  and Lee Hetherington, MIT
 * 7/1992       Added extra menu buttons--Nelson H. F. Beebe <beebe@math.utah.edu>
 * 4/1994	Added DPS support, bounding box
 *					--Ricardo Telichevesky
 *					  and Luis Miguel Silveira, MIT RLE.
 * 2/1995       Added rulers support    --Nelson H. F. Beebe <beebe@math.utah.edu>
 * 1/2001	Added source specials	--including ideas from Stefan Ulrich,
 *					  U Munich
 *
 * Compilation options:
 *
 * VMS		    compile for VMS
 * WORDS_BIGENDIAN  store bitmaps internally with most significant bit first
 * BMTYPE	    store bitmaps in unsigned BMTYPE
 * BMBYTES	    sizeof(unsigned BMTYPE)
 * ALTFONT	    default for -altfont option
 * SHRINK	    default for -s option (shrink factor)
 * MFMODE	    default for -mfmode option
 * A4		    use European size paper, and change default dimension to cm
 * TEXXET	    support reflection dvi codes (right-to-left typesetting)
 * GREY		    use grey levels to shrink fonts
 * PS_GS	    use Ghostscript to render pictures/bounding boxes
 * PS_DPS	    use display postscript to render pictures/bounding boxes
 * PS_NEWS	    use the NeWS server to render pictures/bounding boxes
 * GS_PATH	    path to call the Ghostscript interpreter by
 * HTEX		    hypertex enabled.
 * MAGICK	    use ImageMagick to render (external) image files
 */


#include "xdvi-config.h"
#include "xdvi.h"
#include "version.h"

/* Xlib and Xutil are already included */
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>	/* needed for def. of XtNiconX */

#ifdef HAVE_X11_XMU_EDITRES_H
# include <X11/Xmu/Editres.h>
#endif

/* to allow one common dependency file for Xaw/Motif,
   we always include all the headers and have
   #ifdef MOTIF
   tests inside the headers.
*/
#if defined(NEW_MENU_CREATION) || defined(MOTIF)
#include "menu.h"	
#else
#include "xm_menu.h"	
#include "xaw_menu.h"
#endif /* NEW_MENU_CREATION */
#include "xm_toolbar.h"

#ifdef MOTIF

# include <Xm/Xm.h>
# include <Xm/Frame.h>
# include <Xm/PushB.h>
# include <Xm/MainW.h>
# include <Xm/ToggleB.h>
# include <Xm/RowColumn.h>
# include <Xm/ScrolledW.h>
# include <Xm/MenuShell.h>
# include <Xm/DrawingA.h>
# include <Xm/Form.h>
# include <Xm/PanedW.h>
# include <Xm/List.h>


# ifdef MOTIF11 /* FIXME: We'll probably need a config check for this? */
/* used to set up XmStrings */
XmStringCharSet g_charset = (XmStringCharSet)XmSTRING_DEFAULT_CHARSET;
# else
XmStringCharSet g_charset = XmFONTLIST_DEFAULT_TAG;
# endif

#else /* MOTIF */

# include <X11/Xaw/Viewport.h>
# ifdef HTEX
#  include <X11/Xaw/AsciiText.h>
#  include <X11/Xaw/Box.h>
#  include <X11/Xaw/Command.h>
#  include <X11/Xaw/Dialog.h>
#  include <X11/Xaw/Form.h>
#  include <X11/Xaw/Paned.h>
#  include <X11/Xaw/Scrollbar.h>
# endif
# include <X11/Xaw/Command.h>

#endif /* MOTIF */

#include <signal.h>
#include <math.h>	/* sometimes includes atof() */
#include <stdlib.h>
#include <ctype.h>

#include "xserver-info.h"

#include "kpathsea/c-fopen.h"
#include "kpathsea/c-pathch.h"
#include "kpathsea/c-stat.h"
#include "kpathsea/proginit.h"
#include "kpathsea/progname.h"
#include "kpathsea/tex-file.h"
#include "kpathsea/tex-hush.h"
#include "kpathsea/tex-make.h"
#include "string-utils.h"
#include "kpathsea/c-errno.h"

#include "translations.h"
#include "dvi-init.h"
#include "c-openmx.h"
#include "xicon.h"
#include "x_util.h"
#include "message-window.h"
#include "events.h"
#include "mag.h"
#include "pagesel.h"
#include "dvi-draw.h"
#include "statusline.h"
#include "util.h"
#include "hypertex.h"
#include "xaw_menu.h"
#include "xdvi-debug.h"

#include "pixmaps/time16.xbm"
#include "pixmaps/time16_mask.xbm"
#ifdef VMS 
#  include "pixmaps/hand.xbm"
#  include "pixmaps/hand_mask.xbm"
#endif
#include "pixmaps/magglass.xbm"
#include "pixmaps/magglass_mask.xbm"
#include "pixmaps/drag_vert.xbm"
#include "pixmaps/drag_vert_mask.xbm"
#include "pixmaps/drag_horiz.xbm"
#include "pixmaps/drag_horiz_mask.xbm"
#include "pixmaps/drag_omni.xbm"
#include "pixmaps/drag_omni_mask.xbm"

unsigned int	page_w, page_h;

int offset_x, offset_y;
unsigned int unshrunk_paper_w, unshrunk_paper_h;
unsigned int unshrunk_page_w, unshrunk_page_h;

#ifdef DEBUG
#include<asm/msr.h>
unsigned long time_start=0, time_end=0;
#endif

#ifdef MOTIF
#include <Xm/MwmUtil.h>
#else
/* need to fake it */

/* bit definitions for MwmHints.flags */
#define MWM_HINTS_FUNCTIONS (1L << 0)
#define MWM_HINTS_DECORATIONS (1L << 1)
#define MWM_HINTS_INPUT_MODE (1L << 2)
#define MWM_HINTS_STATUS (1L << 3)

/* bit definitions for MwmHints.decorations */
#define MWM_DECOR_ALL  (1L << 0)
#define MWM_DECOR_BORDER (1L << 1)
#define MWM_DECOR_RESIZEH (1L << 2)
#define MWM_DECOR_TITLE  (1L << 3)
#define MWM_DECOR_MENU  (1L << 4)
#define MWM_DECOR_MINIMIZE (1L << 5)
#define MWM_DECOR_MAXIMIZE (1L << 6)

typedef struct
{
    unsigned long flags;
    unsigned long functions;
    unsigned long decorations;
    long          inputMode;
    unsigned long status;
} PropMotifWmHints;

#define PROP_MOTIF_WM_HINTS_ELEMENTS 5

#endif

char *dvi_name = NULL;		/* fully expanded dvi file name */
char *g_dvi_dir_name = NULL;	/* directory component of dvi file name */
size_t g_dvi_dir_len = 0;	/* length of directory component */

FILE *g_bak_fp;			/* backup file */

time_t dvi_time;		/* last modification time */
unsigned char *dvi_property;	/* for setting in window */
size_t dvi_property_length;	/* length of above, for efficiency */
_Xconst char *prog;
int bak_shrink;		/* last shrink factor != 1 */
Dimension window_w, window_h;
XImage *image;
int backing_store;
int home_x, home_y;
Display *DISP;
Screen *SCRN;
XtAccelerators accels_cr, accels_cr_click;

#ifdef GREY
int		screen_number;
Visual		*our_visual;
unsigned int	our_depth;
Colormap	our_colormap;
#else
# define our_depth	(unsigned int) DefaultDepthOfScreen(SCRN)
# define our_visual	DefaultVisualOfScreen(SCRN)
# define our_colormap	DefaultColormapOfScreen(SCRN)
#endif

GC ruleGC;
GC foreGC;
GC invertedGC;
GC highGC;
GC foreGC2;
GC copyGC;
GC rulerGC;
#ifdef HTEX
GC linkcolorGC;
GC visited_linkcolorGC;
#endif /* HTEX */

Boolean	copy;

Cursor wait_cursor;
Cursor ready_cursor;
Cursor corrupted_cursor;
Cursor link_cursor;
Cursor rule_cursor;
Cursor mag_cursor;
Cursor drag_cursor_h;
Cursor drag_cursor_v;
Cursor drag_cursor_a;
Cursor pause_cursor;

unsigned long cursor_flags = 0;


/* global widgets */
Widget top_level = 0;
Widget vport_widget;
Widget form_widget;
Widget paned;
Widget main_window;
Widget x_bar, y_bar;	/* horizontal and vert. scroll bars */
Widget draw_widget;
#ifdef TEST_SCROLLING
Widget draw_background;
#endif
Widget clip_widget;

#ifdef MOTIF
Widget page_list, page_list_scroll;
static Dimension g_save_shadow_thickness;
#endif

#ifndef MOTIF
static Atom mainDeleteWindow;
static void
handle_delete_message(Widget widget, XtPointer closure, XEvent *event, Boolean *cont)
{
    UNUSED(cont);

    if (event->type == ClientMessage
	/* following is in non-k xdvi - not sure what it means. */
	/* && event->xclient.message_type == XA_WM_PROTOCOLS */
	&& (unsigned)(event->xclient.data.l[0]) == mainDeleteWindow) {
	if (closure != 0)
	    ((XtCallbackProc) closure) (widget, NULL, NULL);
	else
	    xdvi_exit(EXIT_SUCCESS);
    }
}
#endif

#ifdef TEST_SCROLLING
#warning ========== compiling with TEST_SCROLLING ==========
#endif


/* for measuring distance from the ruler */
int g_ruler_pos_x, g_ruler_pos_y;

Boolean ignore_papersize_specials = False;

expert_flag_t expert_flag;

/* Mapping of encoding_t to charset names. Note: Names are all-lowercase,
   for comparison via memicmp(). The first one (index 0) will be used as
   default if the user enters an unknown value.
*/
static struct encoding_mapping_t {
    encoding_t encoding;
    const char *encoding_str;
} encoding_mapping[] = {
    { ISO_8859_1, "iso_8859-1"	},
    { UTF_8,	  "utf-8"	},
    { INVALID_ENC, NULL		}
};



/* #include <X11/Xmd.h>	*/ /* get WORD64 and LONG64 */

#ifndef	ALTFONT
# define ALTFONT    "cmr10"
#endif

#ifndef	SHRINK
# define SHRINK	    8
#endif

#ifndef	MFMODE
# define MFMODE	    NULL
#endif

#undef MKTEXPK
#define MKTEXPK MAKEPK

#if defined(PS_GS) && !defined(GS_PATH)
# define GS_PATH "gs"
#endif

#if A4
# define DEFAULT_PAPER		"a4"
#else
# define DEFAULT_PAPER		"us"
#endif

#ifdef	X_NOT_STDC_ENV
# ifndef atof
extern double atof(_Xconst char *);
# endif
#endif

/*
 * Main application context, used for Xt event processing.
 */
XtAppContext app = NULL;

static Dimension bwidth = 2;

struct _resource resource;

unsigned long debug;

XColor	cr_Color; /* needed for communication with events.c (cursor color) */

struct mg_size_rec mg_size[5] = {
    {200, 150}, {400, 250}, {700, 500}, {1000, 800}, {1200, 1200}
};

struct WindowRec mane = { (Window) 0, 1, 0, 0, 0, 0, MAXDIM, 0, MAXDIM, 0 };

/* currwin is temporary storage except for within redraw() */
struct WindowRec currwin = { (Window) 0, 1, 0, 0, 0, 0, MAXDIM, 0, MAXDIM, 0 };

/*
 * Data for options processing.
 */
static _Xconst char SILENT[] = " ";	/* flag value for usage() */

static _Xconst char SUBST[] = "x";	/* another flag value */

static _Xconst char USAGESTR_END_MARKER[] = "__USAGE_END_MARKER__"; /* end marker */

static _Xconst char *SUBST_VAL[] = { "-mgs[n] <size>" };

static XrmOptionDescRec options[] = {
    {"-s",		".shrinkFactor",	XrmoptionSepArg,	(XPointer)NULL	},
#ifndef	VMS
    {"-S",		".densityPercent",	XrmoptionSepArg,	(XPointer)NULL	},
#endif
    {"-density",	".densityPercent",	XrmoptionSepArg,	(XPointer)NULL	},
#if COLOR
    {"-nocolor",	".color",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+nocolor",	".color",		XrmoptionNoArg,		(XPointer)"on"	},
#endif
#ifdef GREY
    {"-nogrey",		".grey",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+nogrey",		".grey",		XrmoptionNoArg,		(XPointer)"on"	},
    {"-gamma",		".gamma",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-invertedfactor", ".invertedFactor",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-install",	".install",		XrmoptionNoArg,		(XPointer)"on"	},
    {"-noinstall",	".install",		XrmoptionNoArg,		(XPointer)"off"	},
#endif
    {"-rulecolor",	".ruleColor",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-p",		".pixelsPerInch",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-margins",	".Margin",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-sidemargin",	".sideMargin",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-topmargin",	".topMargin",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-offsets",	".Offset",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-xoffset",	".xOffset",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-yoffset",	".yOffset",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-paper",		".paper",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-altfont",	".altFont",		XrmoptionSepArg,	(XPointer)NULL	},
#ifdef MKTEXPK
    {"-nomakepk",	".makePk",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+nomakepk",	".makePk",		XrmoptionNoArg,		(XPointer)"on"	},
#endif
    {"-mfmode",		".mfMode",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-editor",		".editor",		XrmoptionSepArg,	(XPointer)NULL	},
#ifdef T1LIB
    {"-not1lib",	".t1lib",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+not1lib",	".t1lib",		XrmoptionNoArg,		(XPointer)"on"	},
#endif
    {"-sourceposition",	".sourcePosition",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-text-encoding",	".textEncoding",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-unique",		".unique",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+unique",		".unique",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-nofork",		".fork",		XrmoptionNoArg,		(XPointer)"off" },
    {"+nofork",		".fork",		XrmoptionNoArg,		(XPointer)"on"  },
/*     {"-subpixels",	".subPixels",		XrmoptionSepArg,	(XPointer)NULL	}, */
    {"-l",		".listFonts",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+l",		".listFonts",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-watchfile",	".watchFile",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-expertmode",	".expertMode",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-expert",		".expert",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+expert",		".expert",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+statusline",	".statusline",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-statusline",	".statusline",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+useTeXpages",	".useTeXPages",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-useTeXpages",	".useTeXPages",		XrmoptionNoArg,		(XPointer)"on"	},
    {"-mgs",		".magnifierSize1",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-mgs1",		".magnifierSize1",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-mgs2",		".magnifierSize2",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-mgs3",		".magnifierSize3",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-mgs4",		".magnifierSize4",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-mgs5",		".magnifierSize5",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-warnspecials",	".warnSpecials",	XrmoptionNoArg,		(XPointer)"on"	},
    {"+warnspecials",	".warnSpecials",	XrmoptionNoArg,		(XPointer)"off"	},
    {"-hush",		".Hush",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+hush",		".Hush",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-hushchars",	".hushLostChars",	XrmoptionNoArg,		(XPointer)"on"	},
    {"+hushchars",	".hushLostChars",	XrmoptionNoArg,		(XPointer)"off"	},
    {"-hushchecksums",	".hushChecksums",	XrmoptionNoArg,		(XPointer)"on"	},
    {"+hushchecksums",	".hushChecksums",	XrmoptionNoArg,		(XPointer)"off"	},
    {"-hushstdout",	".hushStdout",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+hushstdout",	".hushStdout",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-safer",		".safer",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+safer",		".safer",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-fg",		".foreground",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-foreground",	".foreground",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-bg",		".background",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-background",	".background",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-hl",		".highlight",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-cr",		".cursorColor",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-icongeometry",	".iconGeometry",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-keep",		".keepPosition",	XrmoptionNoArg,		(XPointer)"on"	},
    {"+keep",		".keepPosition",	XrmoptionNoArg,		(XPointer)"off"	},
    {"-copy",		".copy",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+copy",		".copy",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-thorough",	".thorough",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+thorough",	".thorough",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-fullscreen",	".fullscreen",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+fullscreen",	".fullscreen",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-pause",		".pause",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+pause",		".pause",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-pausespecial",	".pauseSpecial",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-wheelunit",	".wheelUnit",		XrmoptionSepArg,	(XPointer)NULL	},
#ifdef PS
    {"-postscript",	".postscript",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-allowshell",	".allowShell",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+allowshell",	".allowShell",		XrmoptionNoArg,		(XPointer)"off"	},
# ifdef	PS_DPS
    {"-nodps",		".dps",			XrmoptionNoArg,		(XPointer)"off"	},
    {"+nodps",		".dps",			XrmoptionNoArg,		(XPointer)"on"	},
# endif
# ifdef	PS_NEWS
    {"-nonews",		".news",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+nonews",		".news",		XrmoptionNoArg,		(XPointer)"on"	},
# endif
# ifdef	PS_GS
    {"-noghostscript",	".ghostscript",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+noghostscript",	".ghostscript",		XrmoptionNoArg,		(XPointer)"on"	},
    {"-nogssafer",	".gsSafer",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+nogssafer",	".gsSafer",		XrmoptionNoArg,		(XPointer)"on"	},
    {"-gsalpha",	".gsAlpha",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+gsalpha",	".gsAlpha",		XrmoptionNoArg,		(XPointer)"off"	},
    {"-interpreter",	".interpreter",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-gspalette",	".palette",		XrmoptionSepArg,	(XPointer)NULL	},
# endif
# ifdef	MAGICK
    {"-magick",		".ImageMagick",		XrmoptionNoArg,		(XPointer)"on"  },
    {"+magick",		".ImageMagick",		XrmoptionNoArg,		(XPointer)"off" },
    {"-magick_cache",	".MagickCache",		XrmoptionSepArg,	(XPointer)NULL	},
# endif
#endif /* PS */
    {"-noscan",		".prescan",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+noscan",		".prescan",		XrmoptionNoArg,		(XPointer)"on"	},
    {"-notempfile",	".tempFile",		XrmoptionNoArg,		(XPointer)"off"	},
    {"+notempfile",	".tempFile",		XrmoptionNoArg,		(XPointer)"on"	},
    {"-dvipspath",	".dvipsPath",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-ps2pdfpath",	".ps2pdfPath",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-debug",		".debugLevel",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-version",	".version",		XrmoptionNoArg,		(XPointer)"on"	},
    {"+version",	".version",		XrmoptionNoArg,		(XPointer)"off"	},
#ifdef HTEX
    {"-linkstyle",	".linkStyle",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-linkcolor",	".linkColor",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-visitedlinkcolor",".visitedLinkColor",	XrmoptionSepArg,	(XPointer)NULL	},
    /* we'd like to have the following order of precedence for the
       browser settings (see browser.c):
          1. browser command-line argument
          2. BROWSER environment variable
          3. WWWBROWSER environment variable
          4. wwwBrowser resource
       So -browser cannot set the browser resource, since that would
       make the two indistinguishable. Therefore we use a different
       (undocumented) resource for the command-line argument:
    */
    {"-browser",	".CMD_wwwBrowser",	XrmoptionSepArg,	(XPointer)NULL	},
    {"-urlbase",	".urlBase",		XrmoptionSepArg,	(XPointer)NULL	},
    {"-anchorposition",	".anchorPosition",	XrmoptionSepArg,	(XPointer)NULL	},
#endif
};


static _Xconst char empty_translations[] = "";

#define	offset(field)	XtOffsetOf(struct _resource, field)
static int base_tick_length = 4;

static char XtRBool3[] = "Bool3";	/* resource for Bool3 */

static XtResource application_resources[] = {
    {"regression", "Regression", XtRBoolean, sizeof(Boolean),
     offset(regression), XtRString, "false"},
    /* used to check whether app-defaults file is out of sync. Initialize
       to a low default value (one before the resource was introduced) */
    {"appDefaultsFileVersion", "AppDefaultsFileVersion", XtRInt, sizeof(int),
     offset(app_defaults_fileversion), XtRImmediate, (XtPointer)20030302 },
    {"shrinkFactor", "ShrinkFactor", XtRInt, sizeof(int),
     offset(shrinkfactor), XtRImmediate, (XtPointer) SHRINK},
    {"delayRulers", "DelayRulers", XtRBoolean, sizeof(Boolean),
     offset(delay_rulers), XtRString, "true"},
    {"useTeXPages", "UseTeXPages", XtRBoolean, sizeof(Boolean),
     offset(use_tex_pages), XtRString, "false"},
    {"densityPercent", "DensityPercent", XtRInt, sizeof(int),
     offset(density), XtRString, "40"},
    {"mainTranslations", "MainTranslations", XtRString, sizeof(char *),
     offset(main_translations), XtRString, (XtPointer) NULL},
    {"wheelTranslations", "WheelTranslations", XtRString, sizeof(char *),
     offset(wheel_translations), XtRString, (XtPointer) "<Btn4Down>:wheel(-0.2)\n<Btn5Down>:wheel(0.2)"},
    {"wheelUnit", "WheelUnit", XtRInt, sizeof(int),
     offset(wheel_unit), XtRImmediate, (XtPointer) 80},
#ifdef GREY
    {"gamma", "Gamma", XtRFloat, sizeof(float),
     offset(gamma), XtRString, "1"},
    {"invertedFactor", "InvertedFactor", XtRFloat, sizeof(float),
     offset(inverted_factor), XtRString, "3.0"},
#endif
    {"pixelsPerInch", "PixelsPerInch", XtRInt, sizeof(int),
     offset(pixels_per_inch), XtRImmediate, (XtPointer) BDPI},
    {"sideMargin", "Margin", XtRString, sizeof(char *),
     offset(sidemargin), XtRString, (XtPointer) NULL},
    {"tickLength", "TickLength", XtRInt, sizeof(int),
     offset(tick_length), XtRInt, (XtPointer) & base_tick_length},
    {"tickUnits", "TickUnits", XtRString, sizeof(char *),
     offset(tick_units), XtRString, "mm"},
    {"topMargin", "Margin", XtRString, sizeof(char *),
     offset(topmargin), XtRString, (XtPointer) NULL},
    {"xOffset", "Offset", XtRString, sizeof(char *),
     offset(xoffset), XtRString, (XtPointer) NULL},
    {"yOffset", "Offset", XtRString, sizeof(char *),
     offset(yoffset), XtRString, (XtPointer) NULL},
    {"paper", "Paper", XtRString, sizeof(char *),
     offset(paper), XtRString, (XtPointer) DEFAULT_PAPER},
    {"paperLandscape", "PaperLandscape", XtRBoolean, sizeof(Boolean),
     offset(paper_landscape), XtRString, "false"},
    {"altFont", "AltFont", XtRString, sizeof(char *),
     offset(alt_font), XtRString, (XtPointer) ALTFONT},
    {"makePk", "MakePk", XtRBoolean, sizeof(Boolean),
     offset(makepk), XtRString,
#ifdef MAKE_TEX_PK_BY_DEFAULT
     "true"
#else
     "false"
#endif
     },
    {"mfMode", "MfMode", XtRString, sizeof(char *),
     offset(mfmode), XtRString, MFMODE},
    {"editor", "Editor", XtRString, sizeof(char *),
     offset(editor), XtRString, (XtPointer) NULL},
#ifdef T1LIB
    {"t1lib", "T1lib", XtRBoolean, sizeof(Boolean),
     offset(t1lib), XtRString, "true"},
#endif
    {"sourcePosition", "SourcePosition", XtRString, sizeof(char *),
     offset(src_pos), XtRString, (XtPointer) NULL},
    {"textEncoding", "TextEncoding", XtRString, sizeof(char *),
     offset(text_encoding), XtRString, (XtPointer) "ISO_8859-1"},
    {"fork", "Fork", XtRBoolean, sizeof(Boolean),
     offset(src_fork), XtRString, "true"},
/*     {"subPixels", "SubPixels", XtRString, sizeof(char *), */
/*      offset(sub_pixels), XtRString, (XtPointer) NULL}, */
    {"unique", "Unique", XtRBoolean, sizeof(Boolean),
     offset(unique), XtRString, "false"},
    {"listFonts", "ListFonts", XtRBoolean, sizeof(Boolean),
     offset(list_fonts), XtRString, "false"},
    {"reverseVideo", "ReverseVideo", XtRBoolean, sizeof(Boolean),
     offset(reverse), XtRString, "false"},
    {"warnSpecials", "WarnSpecials", XtRBoolean, sizeof(Boolean),
     offset(warn_spec), XtRString, "false"},
    {"hushLostChars", "Hush", XtRBoolean, sizeof(Boolean),
     offset(hush_chars), XtRString, "false"},
    {"hushChecksums", "Hush", XtRBoolean, sizeof(Boolean),
     offset(hush_chk), XtRString, "false"},
    {"hushStdout", "HushStdout", XtRBoolean, sizeof(Boolean),
     offset(hush_stdout), XtRString, "false"},
    {"safer", "Safer", XtRBoolean, sizeof(Boolean),
     offset(safer), XtRString, "false"},
#ifdef VMS
    {"foreground", "Foreground", XtRString, sizeof(char *),
     offset(fore_color), XtRString, (XtPointer) NULL},
    {"background", "Background", XtRString, sizeof(char *),
     offset(back_color), XtRString, (XtPointer) NULL},
#endif
    {"iconGeometry", "IconGeometry", XtRString, sizeof(char *),
     offset(icon_geometry), XtRString, (XtPointer) NULL},
    {"keepPosition", "KeepPosition", XtRBoolean, sizeof(Boolean),
     offset(keep_flag), XtRString, "false"},
#ifdef PS
    {"postscript", "Postscript", XtRInt, sizeof(int),
     offset(postscript), XtRImmediate, (XtPointer)1},
    {"allowShell", "AllowShell", XtRBoolean, sizeof(Boolean),
     offset(allow_shell), XtRString, "false"},
# ifdef	PS_DPS
    {"dps", "DPS", XtRBoolean, sizeof(Boolean),
     offset(useDPS), XtRString, "true"},
# endif
# ifdef	PS_NEWS
    {"news", "News", XtRBoolean, sizeof(Boolean),
     offset(useNeWS), XtRString, "true"},
# endif
# ifdef	PS_GS
    {"ghostscript", "Ghostscript", XtRBoolean, sizeof(Boolean),
     offset(useGS), XtRString, "true"},
    {"gsSafer", "Safer", XtRBoolean, sizeof(Boolean),
     offset(gs_safer), XtRString, "true"},
    {"gsAlpha", "Alpha", XtRBoolean, sizeof(Boolean),
     offset(gs_alpha), XtRString, "false"},
    {"interpreter", "Interpreter", XtRString, sizeof(char *),
     offset(gs_path), XtRString, (XtPointer) GS_PATH},
    {"palette", "Palette", XtRString, sizeof(char *),
     offset(gs_palette), XtRString, (XtPointer) "Color"},
# endif /* PS_GS */
# ifdef	MAGICK
    {"magick", "ImageMagick", XtRBoolean, sizeof(Boolean),
     offset(useMAGICK), XtRString, "true"},
    {"magick_cache", "MagickCache", XtRString, sizeof(char *),
     offset(magick_cache), XtRString, (XtPointer) NULL},
# endif
#endif /* PS */
    {"prescan", "Prescan", XtRBoolean, sizeof(Boolean),
     offset(prescan), XtRString, "true"},
    {"tempFile", "TempFile", XtRBoolean, sizeof(Boolean),
     offset(use_temp_fp), XtRString, "true"},
    {"copy", "Copy", XtRBoolean, sizeof(Boolean),
     offset(copy), XtRString, "false"},
    {"thorough", "Thorough", XtRBoolean, sizeof(Boolean),
     offset(thorough), XtRString, "false"},
    {"fullscreen", "Fullscreen", XtRBoolean, sizeof(Boolean),
     offset(fullscreen), XtRString, "false"},
    {"pause", "Pause", XtRBoolean, sizeof(Boolean),
     offset(pause), XtRString, "false"},
    {"pauseSpecial", "PauseSpecial", XtRString, sizeof(char *),
     offset(pause_special), XtRString, (XtPointer)"xdvi:pause"},
    {"debugLevel", "DebugLevel", XtRString, sizeof(char *),
     offset(debug_arg), XtRString, (XtPointer) NULL},
    {"version", "Version", XtRBoolean, sizeof(Boolean),
     offset(version_flag), XtRString, "false"},
    {"menuTranslations", "MenuTranslations", XtRString, sizeof(char *),
     offset(menu_translations), XtRString, (XtPointer) default_menu_config},
    {"watchFile", "WatchFile", XtRFloat, sizeof(float),
     offset(watch_file), XtRString, "0"},
    {"expert", "Expert", XtRBoolean, sizeof(Boolean),
     offset(expert), XtRString, (XtPointer) NULL},
    {"expertMode", "ExpertMode", XtRInt, sizeof(int),
     offset(expert_mode), XtRImmediate, (XtPointer)31 /* everything on */ },
#ifndef MOTIF
    {"buttonSideSpacing", "ButtonSpacing", XtRDimension, sizeof(Dimension),
     offset(btn_side_spacing), XtRImmediate, (XtPointer) 8},
    {"buttonTopSpacing", "ButtonSpacing", XtRDimension, sizeof(Dimension),
     offset(btn_top_spacing), XtRImmediate, (XtPointer) 16},
    {"buttonBetweenSpacing", "ButtonSpacing", XtRDimension, sizeof(Dimension),
     offset(btn_between_spacing), XtRImmediate, (XtPointer) 8},
    /* ignored; only for backwards compatibility */
    {"buttonBetweenExtra", "ButtonSpacing", XtRDimension, sizeof(Dimension),
     offset(btn_between_extra), XtRImmediate, (XtPointer)0},
    {"buttonBorderWidth", "BorderWidth", XtRDimension, sizeof(Dimension),
     offset(btn_border_width), XtRImmediate, (XtPointer) 1},
#endif /* MOTIF */
    {"statusline", "Statusline", XtRBoolean, sizeof(Boolean),
     offset(statusline), XtRString, (XtPointer) NULL},
#ifdef MOTIF
    {"toolbarTranslations", "ToolbarTranslations", XtRString, sizeof(char *),
     offset(toolbar_translations), XtRString, (XtPointer) default_toolbar_translations},
    {"toolbarPixmapFile", "ToolbarPixmapFile", XtRString, sizeof(char *),
     offset(toolbar_pixmap_file), XtRString, (XtPointer) "toolbar.xpm"},
    {"toolbarButtonsRaised", "ToolbarButtonsRaised", XtRBoolean, sizeof(Boolean),
     offset(toolbar_buttons_raised), XtRString, "True"},
#endif /* MOTIF */
    {"pageListHighlightCurrent", "PageListHighlightCurrent", XtRBoolean, sizeof(Boolean),
     offset(pagelist_highlight_current), XtRString, "True"},
    {"pageListWidth", "PageListWidth", XtRDimension, sizeof(Dimension),
     offset(pagelist_width), XtRImmediate, (XtPointer) 80},
    {"magnifierSize1", "MagnifierSize", XtRString, sizeof(char *),
     offset(mg_arg[0]), XtRString, (XtPointer) NULL},
    {"magnifierSize2", "MagnifierSize", XtRString, sizeof(char *),
     offset(mg_arg[1]), XtRString, (XtPointer) NULL},
    {"magnifierSize3", "MagnifierSize", XtRString, sizeof(char *),
     offset(mg_arg[2]), XtRString, (XtPointer) NULL},
    {"magnifierSize4", "MagnifierSize", XtRString, sizeof(char *),
     offset(mg_arg[3]), XtRString, (XtPointer) NULL},
    {"magnifierSize5", "MagnifierSize", XtRString, sizeof(char *),
     offset(mg_arg[4]), XtRString, (XtPointer) NULL},
#if COLOR
    {"color", "Color", XtRBoolean, sizeof(Boolean),
     offset(use_color), XtRString, "true"},
#endif /* COLOR */
    {"dvipsPath", "DvipsPath", XtRString, sizeof(char *),
     offset(dvips_path), XtRString, (XtPointer)DEFAULT_DVIPS_PATH},
    {"ps2pdfPath", "Ps2PdfPath", XtRString, sizeof(char *),
     offset(ps2pdf_path), XtRString, (XtPointer)DEFAULT_PS2PDF_PATH},
    {"dvipsHangTime", "DvipsHangTime", XtRInt, sizeof(int),
     offset(dvips_hang), XtRImmediate, (XtPointer) -1500},
    {"dvipsFailHangTime", "DvipsFailHangTime", XtRInt, sizeof(int),
     offset(dvips_fail_hang), XtRImmediate, (XtPointer) -5000},
    {"dvipsPrinterString", "DvipsPrinterString", XtRString, sizeof(char *),
     offset(dvips_printer_str), XtRString, (XtPointer)NULL},
    {"dvipsOptionsString", "DvipsOptionsString", XtRString, sizeof(char *),
     offset(dvips_options_str), XtRString, (XtPointer)NULL},
#ifdef GREY
    {"grey", "Grey", XtRBoolean, sizeof(Boolean),
     offset(use_grey), XtRString, "true"},
    {"install", "Install", XtRBool3, sizeof(Bool3),
     offset(install), XtRString, "maybe"},
#endif /* GREY */
    {"ruleColor", "RuleColor", XtRPixel, sizeof(Pixel),
     offset(rule_pixel), XtRPixel, (XtPointer) &resource.rule_pixel},
    {"ruleColor", "RuleColor", XtRString, sizeof(char *),
     offset(rule_color), XtRString, (XtPointer) NULL},
#ifdef HTEX
    /* linkStyle: style for presenting links:
       0: no highlighting at all
       1: underline with linkColor
       2: no underlining, text with linkColor
       3: underlining and text with linkColor
    */
    {"linkStyle", "LinkStyle", XtRInt, sizeof(int),
     offset(link_style), XtRImmediate, (XtPointer) 3},
    {"linkColor", "LinkColor", XtRString, sizeof(char *),
     offset(link_color), XtRString, (XtPointer)LINK_COLOR_FALLBACK},
    {"visitedLinkColor", "VisitedLinkColor", XtRString, sizeof(char *),
     offset(visited_link_color), XtRString, (XtPointer)VISITED_LINK_COLOR_FALLBACK},
    {"wwwBrowser", "WWWBrowser", XtRString, sizeof(char *),
     offset(browser), XtRString, (XtPointer) NULL},
    /* the command-line argument -browser sets this undocumented resource */
    {"CMD_wwwBrowser", "CMD_WWWBrowser", XtRString, sizeof(char *),
     offset(CMD_WWWbrowser), XtRString, (XtPointer) NULL},
    /* defaults for unknown mime types */
    {"unknownMimeSuffix", "UnknownMimeSuffix", XtRString, sizeof(char *),
     offset(unknown_mime_suffix), XtRString, "application/x-unknown"},
    {"noMimeSuffix", "NoMimeSuffix", XtRString, sizeof(char *),
     offset(no_mime_suffix), XtRString, "application/x-unknown"},
    {"anchorPosition", "AnchorPosition", XtRString, sizeof(char *),
     offset(anchor_pos), XtRString, (XtPointer) NULL},
#endif /* HTEX */
    /* resources for help text */
    {"helpGeneral", "HelpGeneral", XtRString, sizeof(char *),
     offset(help_general), XtRString, NULL},
#ifdef HTEX
    {"helpHypertex", "HelpHypertex", XtRString, sizeof(char *),
     offset(help_hypertex), XtRString, NULL},
#endif /* HTEX */
    {"helpOthercommands", "HelpOthercommands", XtRString, sizeof(char *),
     offset(help_othercommands), XtRString, NULL},
    {"helpMarking", "HelpMarking", XtRString, sizeof(char *),
     offset(help_marking), XtRString, NULL},
    {"helpPagemotion", "HelpPagemotion", XtRString, sizeof(char *),
     offset(help_pagemotion), XtRString, NULL},
    {"helpMousebuttons", "HelpMousebuttons", XtRString, sizeof(char *),
     offset(help_mousebuttons), XtRString, NULL},
    {"helpRulermode", "HelpRulermode", XtRString, sizeof(char *),
     offset(help_rulermode), XtRString, NULL},
    {"helpSearch", "HelpSearch", XtRString, sizeof(char *),
     offset(help_search), XtRString, NULL},
    {"helpSourcespecials", "HelpSoucespecials", XtRString, sizeof(char *),
     offset(help_sourcespecials), XtRString, NULL},
#ifdef GREY
};

static XtResource app_pixel_resources[] = {	/* get these later */
#endif /* GREY */
    {"foreground", "Foreground", XtRPixel, sizeof(Pixel),
     offset(fore_Pixel), XtRString, XtDefaultForeground},
    {"background", "Background", XtRPixel, sizeof(Pixel),
     offset(back_Pixel), XtRString, XtDefaultBackground},
    {"borderColor", "BorderColor", XtRPixel, sizeof(Pixel),
     offset(brdr_Pixel), XtRPixel, (XtPointer) &resource.fore_Pixel},
    {"highlight", "Highlight", XtRPixel, sizeof(Pixel),
     offset(hl_Pixel), XtRPixel, (XtPointer) &resource.fore_Pixel},
    {"cursorColor", "CursorColor", XtRPixel, sizeof(Pixel),
     offset(cr_Pixel), XtRPixel, (XtPointer) &resource.fore_Pixel},
};
#undef offset

/* TODO: what does the `^' stand for? */
static _Xconst char *usagestr[] = {
    /* shrinkFactor	*/ "shrink",
#ifndef	VMS
    /* S		*/ "density",
    /* density		*/ SILENT,
#else
    /* density		*/ "density",
#endif
#ifdef	GREY
    /* gamma		*/ "<float>",
    /* highlightfactor	*/ "<float>",
#endif
    /* rulecolor	*/ "color",
    /* p		*/ "pixels",
    /* margins		*/ "dimen",
    /* sidemargin	*/ "dimen",
    /* topmargin	*/ "dimen",
    /* offsets		*/ "dimen",
    /* xoffset		*/ "dimen",
    /* yoffset		*/ "dimen",
    /* paper		*/ "papertype",
    /* altfont		*/ "font",
    /* mfmode		*/ "mode-def",
    /* editor		*/ "editor",
    /* sourceposition	*/ "linenumber[ ]*filename",
    /* textencoding	*/ "charset",
    /* rv		*/ "^-l", "-rv",
    /* watchfile	*/ "secs",
    /* expertmode	*/ "flag",
    /* mgs		*/ SUBST,
    /* mgs1		*/ SILENT,
    /* mgs2		*/ SILENT,
    /* mgs3		*/ SILENT,
    /* mgs4		*/ SILENT,
    /* mgs5		*/ SILENT,
    /* bw		*/ "^-safer", "-bw <width>",
    /* fg		*/ "color",
    /* foreground	*/ SILENT,
    /* bg		*/ "color",
    /* background	*/ SILENT,
    /* hl		*/ "color",
    /* bd		*/ "^-hl", "-bd <color>",
    /* cr		*/ "color",
#ifndef VMS
    /* display		*/ "^-cr", "-display <host:display>",
#else
    /* display		*/ "^-cr", "-display <host::display>",
#endif
    /* geometry		*/ "^-cr", "-geometry <geometry>",
    /* icongeometry	*/ "geometry",
    /* iconic		*/ "^-icongeometry", "-iconic",
    /* font		*/ "^-icongeometry", "-font <font>",
    /* pausespecial	*/ "string",
    /* wheelunit	*/ "pixels",
#ifdef PS
    /* postscript	*/ "0|1|2",
# ifdef PS_GS
    /* interpreter	*/ "path",
    /* gspalette	*/ "monochrome|grayscale|color",
# endif
# ifdef MAGICK
    /* magick_cache	*/ "size[k|K|m|M|g|G]",
# endif
#endif
    /* dvipspath	*/ "path",
    /* ps2pdfpath	*/ "path",
    /* debug		*/ "bitmask|string",
#ifdef HTEX
    /* linkstyle	*/ "0|1|2|3",
    /* linkcolor	*/ "color",
    /* visitedlinkcolor	*/ "color",
    /* browser		*/ "WWWbrowser",
    /* URLbase		*/ "base URL",
    /* anchorposition	*/ "anchor",
#endif
    /* [end marker]	*/ USAGESTR_END_MARKER
};

#ifndef MOTIF

# ifdef	NOQUERY
#  define drawWidgetClass widgetClass
# else

static XtGeometryResult
QueryGeometry(Widget w,
	      XtWidgetGeometry *constraints,
	      XtWidgetGeometry *reply)
{
    UNUSED(w);
    UNUSED(constraints);
    reply->request_mode = CWWidth | CWHeight;
    reply->width = page_w;
    reply->height = page_h + global_statusline_h;

    return XtGeometryAlmost;
}

# include <X11/IntrinsicP.h>
# include <X11/CoreP.h>

/* if the following gives you trouble, just compile with -DNOQUERY */
static WidgetClassRec drawingWidgetClass = {
    {
     /* superclass		*/ &widgetClassRec,
     /* class_name		*/ "Draw",
     /* widget_size		*/ sizeof(WidgetRec),
     /* class_initialize	*/ NULL,
     /* class_part_initialize	*/ NULL,
     /* class_inited		*/ FALSE,
     /* initialize		*/ NULL,
     /* initialize_hook		*/ NULL,
     /* realize			*/ XtInheritRealize,
     /* actions			*/ NULL,
     /* num_actions		*/ 0,
     /* resources		*/ NULL,
     /* num_resources		*/ 0,
     /* xrm_class		*/ NULLQUARK,
     /* compress_motion		*/ FALSE,
     /* compress_exposure	*/ TRUE,
     /* compress_enterleave	*/ FALSE,
     /* visible_interest	*/ FALSE,
     /* destroy			*/ NULL,
     /* resize			*/ XtInheritResize,
     /* expose			*/ XtInheritExpose,
     /* set_values		*/ NULL,
     /* set_values_hook		*/ NULL,
     /* set_values_almost	*/ XtInheritSetValuesAlmost,
     /* get_values_hook		*/ NULL,
     /* accept_focus		*/ XtInheritAcceptFocus,
     /* version			*/ XtVersion,
     /* callback_offsets	*/ NULL,
     /* tm_table		*/ XtInheritTranslations,
     /* query_geometry		*/ QueryGeometry,
     /* display_accelerator	*/ XtInheritDisplayAccelerator,
     /* extension		*/ NULL
     }
};

#  define drawWidgetClass &drawingWidgetClass

# endif /* NOQUERY */
#endif /* not MOTIF */

static int
compare_strings(s,t)
_Xconst void *s;
_Xconst void *t;
{
    const char **ss = (const char**)s;
    const char **tt = (const char**)t;

    return memicmp(*ss, *tt, strlen(*tt) + 1); /* also check for final 0 */
}

static void
usage(int exitval)
{
    XrmOptionDescRec *opt;
    _Xconst char **usageptr = usagestr;
    _Xconst char **sv = SUBST_VAL;
    _Xconst char *str1;
    _Xconst char *str2;
    const char *sorted_options[XtNumber(options)];
    char buf[256];
    char *s;
    int col, n;
    size_t nopt = 0, k;
    
    for (opt = options; opt < options + XtNumber(options); ++opt) {
	str1 = opt->option;
	if (*str1 != '-')
	    continue;

	ASSERT(*usageptr != USAGESTR_END_MARKER, "Too few elements in usageptr[]");

	str2 = NULL;
	if (opt->argKind != XrmoptionNoArg) {
	    str2 = *usageptr++;
	    if (str2 == SILENT)
		continue;
	    if (str2 == SUBST) {
		str1 = *sv++;
		str2 = NULL;
	    }
	}
#if 0
	fprintf(stderr, "str1: %s, str2: %s\n", str1, str2);
#endif
	for (;;) {
	    if (str2 == NULL)
		sprintf(buf, "[%.80s]", str1);
	    else
		sprintf(buf, "[%.80s <%.80s>]", str1, str2);

/* 	    fprintf(stderr, "number of options: %d; len of usagestr: %d\n", */
/* 		    XtNumber(options), XtNumber(usagestr)); */
	    ASSERT(nopt < XtNumber(options), "index out of range");
/* 	    fprintf(stderr, "sorted: %d=%s\n", nopt, buf); */
	    sorted_options[nopt++] = xstrdup(buf);
	    
	    if (**usageptr != '^' || strcmp(*usageptr + 1, opt->option) != 0)
		break;
	    ++usageptr;
	    str1 = *usageptr++;
	    str2 = NULL;
	}
    }

    ASSERT(*usageptr == USAGESTR_END_MARKER, "Too many elements in usageptr[]");

/*     fprintf(stderr, "elems in sorted options: %d\n", nopt); */
    qsort((void*)sorted_options,
	  nopt,
	  sizeof(sorted_options[0]),
	  compare_strings);
    
    s = xstrdup("Usage: ");
    s = xstrcat(s, prog);
    s = xstrcat(s, " [+[<page>]]");

    col = strlen(s);
    fputs(s, stdout);
    
    for (k = 0; k < nopt; ++k) {
	n = strlen(sorted_options[k]);
	if (col + n < 80)
	    putc(' ', stdout);
	else {
	    fputs("\n\t", stdout);
	    col = 8 - 1;
	}
	fputs(sorted_options[k], stdout);
	col += n + 1;
    }

    /* put this in an extra line, to emphasize that it must come last */
#ifdef SELFILE
    fputs("\n\t[dvi_file]\n", stdout);
#else
    fputs("\n\tdvi_file\n", stdout);
#endif

    exit(exitval);
}

int
atopix(_Xconst char *arg, Boolean allow_minus)
{
    int len = strlen(arg);
    _Xconst char *arg_end = arg;
    char tmp[11];
    double factor;

    if (allow_minus && *arg_end == '-')
	++arg_end;
    while ((*arg_end >= '0' && *arg_end <= '9') || *arg_end == '.')
	if (arg_end >= arg + XtNumber(tmp) - 1)
	    return 0;
	else
	    ++arg_end;
    bcopy(arg, tmp, arg_end - arg);
    tmp[arg_end - arg] = '\0';

#if A4
    factor = 1.0 / 2.54;	/* cm */
#else
    factor = 1.0;	/* inches */
#endif
    if (len > 2)
	switch (arg[len - 2] << 8 | arg[len - 1]) {
#if A4
	case 'i' << 8 | 'n':
	    factor = 1.0;
	    break;
#else
	case 'c' << 8 | 'm':
	    factor = 1.0 / 2.54;
	    break;
#endif
	case 'm' << 8 | 'm':
	    factor = 1.0 / 25.4;
	    break;
	case 'p' << 8 | 't':
	    factor = 1.0 / 72.27;
	    break;
	case 'p' << 8 | 'c':
	    factor = 12.0 / 72.27;
	    break;
	case 'b' << 8 | 'p':
	    factor = 1.0 / 72.0;
	    break;
	case 'd' << 8 | 'd':
	    factor = 1238.0 / 1157.0 / 72.27;
	    break;
	case 'c' << 8 | 'c':
	    factor = 12 * 1238.0 / 1157.0 / 72.27;
	    break;
	case 's' << 8 | 'p':
	    factor = 1.0 / 72.27 / 65536;
	    break;
	}

    return factor * atof(tmp) * resource.pixels_per_inch + 0.5;
}

#if CHECK_APP_FILEVERSION
static void
check_app_defaults_fileversion(void)
{
    /* update this when new essential resources are introduced */
    static const int required_version = 20030303;
    
    if (resource.app_defaults_fileversion < required_version) {
	const char *filename = kpse_find_file("XDvi", kpse_program_text_format, 0);
	if (filename == NULL)
	    filename = "XDvi";

	fprintf(stderr, "filename: %d\n", resource.app_defaults_fileversion);
	popup_message(MSG_WARN,
			 NULL,
			 "Your application defaults file `%s' is outdated. "
			 "This version of xdvi requires a file "
			 "with date >= %d. Your file is older, or doesn't have a date.\n"
			 "Please obtain a new version of the file `XDvi', e.g. from:\n"
			 "http://cvs.sourceforge.net/cgi-bin/viewcvs.cgi/xdvi/xdvik/texk/xdvik/texmf/\n"
			 "and copy it to a directory in your $XDVIINPUTS path.",
			 filename, required_version);
    }
}
#endif


static void
warn_about_prerelease_versions(void)
{
    int unstable_version = 0;
    if (strstr(XDVI_VERSION_INFO, "-cvs") != NULL)
	unstable_version = 1;
    else if (strstr(XDVI_VERSION_INFO, "-beta") != NULL)
	unstable_version = 2;

    if (unstable_version > 0) {
	printf("\n**********************************************************************\n");
	printf("This is %s version %s,\n%s version.\n\n", XDVI_PROGNAME, XDVI_VERSION_INFO,
	       unstable_version == 1 ? "an unstable development" : "a beta testing");
	printf("Want a stable version instead?\n"
	       " -> please visit one of:\n"
	       "    http://xdvi.sourceforge.net/cvs-upgrade.html\n"
	       "    http://sourceforge.net/project/showfiles.php?group_id=23164\n\n"
	       "Found a bug?\n"
	       " -> please report it to:\n"
	       "    http://sourceforge.net/tracker/?group_id=23164&atid=377580\n\n"
	       "Thanks for your support!\n");
	printf("**********************************************************************\n");
    }
}


/*
 * Main programs start here.
 */

#ifdef GREY
static Arg temp_args1[] = {
    {XtNdepth, (XtArgVal) 0},
    {XtNvisual, (XtArgVal) 0},
    {XtNcolormap, (XtArgVal) 0},
};

/*
 * Alternate routine to convert color name to Pixel (needed to substitute
 * "black" or "white" for BlackPixelOfScreen, etc., since a different visual
 * and colormap are in use).
 */

static Boolean
XdviCvtStringToPixel(Display *dpy,
		     XrmValuePtr args, Cardinal *num_args,
		     XrmValuePtr fromVal, XrmValuePtr toVal,
		     XtPointer *closure_ret)
{
    XrmValue replacement_val;
    Boolean default_is_fg;

    if ((strcmp((String) fromVal->addr, XtDefaultForeground) == 0
	 && (default_is_fg = True, True))
	|| (strcmp((String) fromVal->addr, XtDefaultBackground) == 0
	    && ((default_is_fg = False), True))) {
	replacement_val.size = sizeof(String);
	replacement_val.addr = (default_is_fg == resource.reverse)
	    ? "white" : "black";
	fromVal = &replacement_val;
    }

    return XtCvtStringToPixel(dpy, args, num_args, fromVal, toVal, closure_ret);
}

/*
 * Convert string to yes/no/maybe.  Adapted from the X toolkit.
 */

static Boolean
XdviCvtStringToBool3(Display *dpy,
		     XrmValuePtr args, Cardinal *num_args,
		     XrmValuePtr fromVal, XrmValuePtr toVal,
		     XtPointer *closure_ret)
{
    String str = (String) fromVal->addr;
    static Bool3 value;

    UNUSED(args);
    UNUSED(num_args);
    UNUSED(closure_ret);
    
    if (memicmp(str, "true", 5) == 0
	|| memicmp(str, "yes", 4) == 0
	|| memicmp(str, "on", 3) == 0 || memicmp(str, "1", 2) == 0)
	value = True;

    else if (memicmp(str, "false", 6) == 0
	     || memicmp(str, "no", 3) == 0
	     || memicmp(str, "off", 4) == 0 || memicmp(str, "0", 2) == 0)
	value = False;

    else if (memicmp(str, "maybe", 6) == 0)
	value = Maybe;

    else {
	XtDisplayStringConversionWarning(dpy, str, XtRBoolean);
	return False;
    }

    if (toVal->addr != NULL) {
	if (toVal->size < sizeof(Bool3)) {
	    toVal->size = sizeof(Bool3);
	    return False;
	}
	*(Bool3 *) (toVal->addr) = value;
    }
    else
	toVal->addr = (XPointer) & value;

    toVal->size = sizeof(Bool3);
    return True;
}

#endif

/*
 *	Routines for running as source-special client.
 */

static int XdviErrorHandler P2H(Display *, XErrorEvent *);

static unsigned long xdvi_next_request = 0;
static int xerrno;
static int (*XdviOldErrorHandler)(Display *, XErrorEvent *);

static int XdviErrorHandler(Display *d, XErrorEvent *event)
{
    if (event->serial != xdvi_next_request || event->error_code != BadWindow)
	return XdviOldErrorHandler(d, event);

    xerrno = 1;
    return 0;
}

static int
XdviGetWindowProperty(Display *display,
		      Window w,
		      Atom property,
		      long long_offset,
		      long long_length,
		      Bool delete,
		      Atom req_type,
		      Atom *actual_type_return,
		      int *actual_format_return,
		      unsigned long *nitems_return,
		      unsigned long *bytes_after_return,
		      unsigned char **prop_return)
{
    int retval;

    xdvi_next_request = NextRequest(display);
    xerrno = 0;

    retval = XGetWindowProperty(display, w, property, long_offset,
				long_length, delete, req_type,
				actual_type_return, actual_format_return,
				nitems_return, bytes_after_return, prop_return);

    return (xerrno != 0 ? BadWindow : retval);
}

/* helper function to set a string property of type `prop' for window `win' */
static void
set_string_property(const char *str, Atom prop, Window win)
{
    XChangeProperty(DISP, win, prop, prop, 8, PropModeReplace,
		    (_Xconst unsigned char *)str, strlen(str));
    XFlush(DISP);	/* necessary to get the property set */
}

/*
 * Set the `sourceposition' propery of the window `w' to `source_str'.
 * This is usually done in `client' mode to notify the instance of
 * xdvi running in window `w' to start a forward search. After this,
 * the client usually exits normally.
 */
static void
set_sourceposition_property(const char *source_str, Window win)
{
    /* parse the special in order to expand the filename */
    struct src_parsed_special data;
    char *new_special = NULL;
    char *expanded_filename = NULL;
	    
    data.filename_len = 0;
    data.filename = NULL;
	    
    src_parse(source_str, &data);

    if (data.filename_len == 0) {
	/* can't give a GUI warning in `client' mode - just exit with error */
	XDVI_FATAL((stderr,
		    "Filename missing in -sourceposition argument (%s)!",
		    source_str));
    }
	    
    TRACE_CLIENT((stderr, "got data: line %d, col %d, file |%s|, len %d\n",
		  data.line, data.col, data.filename, data.filename_len));

    /* expand -sourceposition argument if it contains a path component.
       We don't use REALPATH here, because `tex -src' doesn't expand
       symlinks either. Instead, use canonicalize_path() to expand
       `../' and './' manually. */
    if (strchr(data.filename, '/') != NULL
	&& (expanded_filename = expand_filename(data.filename, USE_CWD_PATH)) != NULL) {
	char *tmp = canonicalize_path(expanded_filename);
	free(data.filename);
	free(expanded_filename);
	expanded_filename = tmp;		
    }
    else
	expanded_filename = data.filename;
	    
    TRACE_CLIENT((stderr, "expanded1: |%s|\n", expanded_filename));

    new_special = xmalloc(2 * LENGTH_OF_INT + 2 /* 2 for `:' and separating space */
			  + strlen(expanded_filename) + 1);
    sprintf(new_special, "%d:%d %s", data.line, data.col, expanded_filename);
    free(expanded_filename);				  
				  
    TRACE_CLIENT((stderr, "matched!"));
    set_string_property(new_special, atom_src_goto(), win);
    free(new_special);
    set_string_property("", atom_raise(), win);
}

/*
 * Check for another running copy of xdvi. If same_file is true, return
 * the window ID of that other instance only if it has currently loaded the
 * same file; else, return 0.
 * If same_file is false, return the first valid xdvi window ID.
 */
static Window
get_xdvi_window_id(Boolean same_file)
{
    unsigned char *window_list;
    size_t window_list_len;
    unsigned char *window_list_end;
    unsigned char *wp;
    unsigned char *p;
    Boolean need_rewrite = False;
    Window ret_window = 0;

    /*
     * Get window list.  Copy it over (we'll be calling property_get_data()
     * again).
     */
    if ((window_list_len = property_get_window_list(&p)) == 0)
	return False;

    window_list = xmalloc(window_list_len);
    memcpy(window_list, p, window_list_len);

    XdviOldErrorHandler = XSetErrorHandler(XdviErrorHandler);

    /* Loop over list of windows.  */

    window_list_end = window_list + window_list_len;
    TRACE_CLIENT((stderr, "My property: `%s'", dvi_property));

    for (wp = window_list; wp < window_list_end; wp += 4) {
	Window w;
	unsigned char *buf_ret;
	size_t len;

	w = get_window_id(wp);

	TRACE_CLIENT((stderr, "Checking window %08lx", w));
	
	len = property_get_data(w, atom_dvi_file(), &buf_ret,
				XdviGetWindowProperty);

	if (len == 0) {
	    /* not getting back info for a window usually indicates
	       that the application the window had belonged to had
	       been killed with signal 9
	    */
	    TRACE_CLIENT((stderr, "Window %08lx: doesn't exist any more, deleting", w));
	    window_list_len -= 4;
	    window_list_end -= 4;
	    bcopy(wp + 4, wp, window_list_end - wp);
	    wp -= 4;
	    need_rewrite = True;
	    continue;
	}
	else { /* window still alive */
	    if (debug & DBG_CLIENT) {
#if 0
		unsigned long ino;
		int i;
		
		ino = 0;
		for (i = 7; i >= 0; --i)
		    ino = (ino << 8) | buf_ret[i];
#endif
		TRACE_CLIENT((stderr, "Window %08lx: property: `%s'", w, buf_ret));
	    }
	    if (!same_file) {
		ret_window = w;
		break;
	    }
	    else if (strcmp((char *)buf_ret, (char *)dvi_property) == 0) { /* match */
		ret_window = w;
		break;
	    }
	}
    }

    XSetErrorHandler(XdviOldErrorHandler);

    if (need_rewrite)
	XChangeProperty(DISP, DefaultRootWindow(DISP),
			atom_xdvi_windows(), atom_xdvi_windows(), 32,
			PropModeReplace, (unsigned char *)window_list,
			window_list_len / 4);

    return ret_window;
}


/* Translation of valid paper types to dimensions,
   which are used internally. The newline characters are a hack
   to format the list neatly for error messages.
*/
static _Xconst char *paper_types[] = {
    "us", "8.5x11in",
    "usr", "11x8.5in",
    "legal", "8.5x14in",
    "legalr", "14x8.5in",
    "foolscap", "13.5x17.0in",	/* ??? */
    "foolscapr", "17.0x13.5in",
    "\n", "0",

    /* ISO `A' formats, Portrait */
    "a1", "594x840mm",
    "a2", "420x594mm",
    "a3", "297x420mm",
    "a4", "210x297mm",
    "a5", "148x210mm",
    "a6", "105x148mm",
    "a7", "74x105mm",
    "\n", "0",

    /* ISO `A' formats, Landscape */
    "a1r", "840x594mm",
    "a2r", "594x420mm",
    "a3r", "420x297mm",
    "a4r", "297x210mm",
    "a5r", "210x148mm",
    "a6r", "148x105mm",
    "a7r", "105x74mm",
    "\n", "0",

    /* ISO `B' formats, Portrait */
    "b1", "707x1000mm",
    "b2", "500x707mm",
    "b3", "353x500mm",
    "b4", "250x353mm",
    "b5", "176x250mm",
    "b6", "125x176mm",
    "b7", "88x125mm",
    "\n", "0",

    /* ISO `B' formats, Landscape */
    "b1r", "1000x707mm",
    "b2r", "707x500mm",
    "b3r", "500x353mm",
    "b4r", "353x250mm",
    "b5r", "250x176mm",
    "b6r", "176x125mm",
    "b7r", "125x88mm",
    "\n", "0",

    /* ISO `C' formats, Portrait */
    "c1", "648x917mm",
    "c2", "458x648mm",
    "c3", "324x458mm",
    "c4", "229x324mm",
    "c5", "162x229mm",
    "c6", "114x162mm",
    "c7", "81x114mm",
    "\n", "0",

    /* ISO `C' formats, Landscape */
    "c1r", "917x648mm",
    "c2r", "648x458mm",
    "c3r", "458x324mm",
    "c4r", "324x229mm",
    "c5r", "229x162mm",
    "c6r", "162x114mm",
    "c7r", "114x81mm",
};

static Boolean
set_paper_type(_Xconst char *arg)
{
    _Xconst char *arg1;
    char temp[21];
    _Xconst char **p;
    char *q;

    if (*arg == '+') {
	++arg;
	ignore_papersize_specials = True;
    }
    if (strlen(arg) > sizeof(temp) - 1)
	return False;
    q = temp;
    for (;;) {	/* convert to lower case */
	char c = *arg++;
	if (c >= 'A' && c <= 'Z')
	    c ^= ('a' ^ 'A');
	*q++ = c;
	if (c == '\0')
	    break;
    }
    arg = temp;
    /* perform substitutions */
    for (p = paper_types; p < paper_types + XtNumber(paper_types); p += 2)
	if (strcmp(temp, *p) == 0) {
	    arg = p[1];
	    break;
	}
    arg1 = strchr(arg, 'x');
    if (arg1 == NULL)
	return False;
    unshrunk_paper_w = atopix(arg, False);
    unshrunk_paper_h = atopix(arg1 + 1, False);

    return (unshrunk_paper_w != 0 && unshrunk_paper_h != 0);
}

/* Set the icon name and title name standard properties on `top_level'.
 * We use the basename of the DVI file (without the .dvi), so different xdvi
 * invocations can be distinguished, yet do not use up too much real estate.
 *
 * This function returns freshly allocated memory in *icon_name and *title_name
 * which the caller is responsible for free()ing again.
 */
void
get_icon_and_title(char *filename, char **icon_name, char **title_name)
{
    /* Use basename of DVI file for name in icon and title.  */
    char *ptr;

    /* SU 2000/12/16: added page number information */
    _Xconst char *_Xconst title_name_fmt = "%s:  %s   (%d page%s)";

    MYTRACE((stderr, "get_icon_and_title called with: |%s|", filename));

    ptr = strrchr(filename, '/');
    if (ptr != NULL)
	++ptr;
    else {
	ptr = filename;
    }
        
    /*
     * Remove the `file:' prefix from the icon name; since some windowmanagers
     * only display a prefix in window lists etc., it's more significant this
     * way.
     */

    if (memcmp(ptr, "file:", 5) == 0) {
	ptr += 5;
    }
    *icon_name = xstrdup(ptr);
    
    MYTRACE((stderr, "before chopping: icon_name: |%s|", *icon_name));
    if ((ptr = strstr(*icon_name, ".dvi")) != NULL) {
	/* chop off .dvi extension */
	*ptr = '\0';
    }
    MYTRACE((stderr, "after chopping: icon_name: |%s|", *icon_name));

    *title_name = xmalloc(strlen(title_name_fmt)
			  + strlen(XDVI_PROGNAME)
			  + strlen(*icon_name)
			  + LENGTH_OF_INT
			  + 2);	/* 2 for additional plural `s' */
    MYTRACE((stderr, "total_pages: %d", total_pages));
    sprintf(*title_name, title_name_fmt, XDVI_PROGNAME, *icon_name, total_pages,
	    (total_pages > 1) ? "s" : "");

    MYTRACE((stderr, "title_name, icon_name: |%s|%s|", *title_name, *icon_name));
}

void
set_icon_and_title(const char *icon_name, const char *title_name)
{
    Window top_window;
    if (!XtIsRealized(top_level)) {
	MYTRACE((stderr, "set_icon_and_title: returning"));
	return;
    }
    top_window = XtWindow(top_level);
    XtVaSetValues(top_level, XtNtitle, (XtArgVal) title_name, XtNiconName, (XtArgVal) icon_name, NULL);
    XSetStandardProperties(DISP, top_window, title_name, icon_name,
			   (Pixmap) 0, NULL, 0, NULL);
}

static void
display_version_info(void)
{
    printf("This is %s version %s\n(%s)\n", XDVI_PROGNAME, XDVI_VERSION_INFO, kpathsea_version_string);
#ifdef MOTIF
    printf("Compiled with %s, runtime version %d.%d\n",
	   /* 	   XmVERSION, XmREVISION, XmUPDATE_LEVEL, */
	   XmVERSION_STRING,
	   xmUseVersion / 1000, xmUseVersion % 1000);
#endif
    printf("\n");
}

static void
display_bug_reporting_info(void)
{
    printf("Please send bug reports, feature requests etc. to one of:\n"
	   "   http://sourceforge.net/tracker/?group_id=23164&atid=377580\n"
	   "   tex-k@tug.org (http://tug.org/mailman/listinfo/tex-k)\n\n"
	   "\n");
}

static void
display_licensing_info(void)
{
    printf("Xdvik is licensed under the X Consortium license.\n"
	   "It additionally uses the following libraries:\n"
	   "- The kpathsea library, licensed under the GNU Library General Public\n"
	   "  License (see COPYING.LIB for full details)\n"
	   "- t1lib, licensed in parts under the GNU Library General Public License,\n"
	   "  in parts under the X Consortium license.\n"
	   "There is NO WARRANTY of anything.\n\n"
	   );
}

void
set_windowsize(Dimension *ret_w, Dimension *ret_h, int add_w, int add_h, Boolean override)
{
    static Arg set_wh_args[] = {
	{XtNwidth, (XtArgVal) 0},
	{XtNheight, (XtArgVal) 0},
    };
    Dimension screen_w, screen_h;
    
    if (resource.fullscreen) {
	Dimension w = WidthOfScreen(SCRN), h = HeightOfScreen(SCRN);

	if (currwin.shrinkfactor == 0) { /* if not set by user */
	    Dimension height_factor = ROUNDUP(unshrunk_page_h, h);
	    currwin.shrinkfactor = ROUNDUP(unshrunk_page_w, w);
	    if (height_factor >= currwin.shrinkfactor)
		currwin.shrinkfactor = height_factor;
/*  	    fprintf(stderr, "factor was 0, using %d\n", currwin.shrinkfactor); */
	}
/*  	else */
/*  	    fprintf(stderr, "factor != 0, using %d\n", currwin.shrinkfactor); */

	/* save current shrinkfactor for later use */
	if (currwin.shrinkfactor > 1)
	    bak_shrink = currwin.shrinkfactor;
	
	mane.shrinkfactor = currwin.shrinkfactor;
	init_page();
	set_wh_args[0].value = (XtArgVal)w;
	set_wh_args[1].value = (XtArgVal)h;
	*ret_w = w;
	*ret_h = h;
#ifdef MOTIF
	XtVaGetValues(main_window, XmNshadowThickness, &g_save_shadow_thickness, NULL);
	XtVaSetValues(main_window, XmNshadowThickness, 0, NULL);
#endif
	XtSetValues(top_level, set_wh_args, XtNumber(set_wh_args));
    }
    else if (override) {
	set_wh_args[0].value = (XtArgVal)*ret_w;
	set_wh_args[1].value = (XtArgVal)*ret_h;
	XtSetValues(top_level, set_wh_args, XtNumber(set_wh_args));
    }
    else { /* determine a window size that fits the current shrink factor */
	XtWidgetGeometry constraints;
	XtWidgetGeometry reply;

	Arg temp_args3 = { XtNborderWidth, (XtArgVal)&bwidth };

	XtGetValues(top_level, &temp_args3, 1);	/* get border width */
	screen_w = WidthOfScreen(SCRN) - 2 * bwidth;

	screen_w -= add_w;
	screen_h = HeightOfScreen(SCRN) - 2 * bwidth - global_statusline_h - 6;
	for (;;) {	/* actually, at most two passes */
	    Dimension height_factor;

	    constraints.request_mode = reply.request_mode = 0;

/*  	    fprintf(stderr, "setting constraints.width to %d\n", page_w); */
	    constraints.width = page_w;
	    if (page_w > screen_w) {
/*  		fprintf(stderr, "setting CWWidth constraint\n"); */
		constraints.request_mode |= CWWidth;
		constraints.width = screen_w;
	    }

/*  	    fprintf(stderr, "setting constraints.height to %d\n", page_h + global_statusline_h); */
	    constraints.height = page_h + global_statusline_h;
	    if (constraints.height > screen_h) {
/*  		fprintf(stderr, "setting CWHeight constraint\n"); */
		constraints.request_mode |= CWHeight;
		/* if window too large, need to subtract height of toolbar etc. again */
		constraints.height = screen_h - add_h;
	    }

	    if (constraints.request_mode != 0
		&& constraints.request_mode != (CWWidth | CWHeight)) {
/*  		fprintf(stderr, "querying geometry!\n"); */
#ifdef MOTIF
		(void)XtQueryGeometry(main_window, &constraints, &reply);
#else
		(void)XtQueryGeometry(vport_widget, &constraints, &reply);
#endif
	    }
	    if (!(reply.request_mode & CWWidth))
		reply.width = constraints.width;
	    if (reply.width >= screen_w)
		reply.width = screen_w;
	    if (!(reply.request_mode & CWHeight))
		reply.height = constraints.height;
	    if (reply.height >= screen_h)
		reply.height = screen_h;

/*  	    fprintf(stderr, "reply: w=%d, h=%d\n", reply.width, reply.height); */
	    /* now reply.{width,height} contain max. usable window size */

	    /* User didn't use `-s 0', use either default or other user-specified value */
	    if (currwin.shrinkfactor != 0) {
/*  		fprintf(stderr, "factor != 0, using %d\n", currwin.shrinkfactor); */
		break;
	    }
/*  	    else { */
/*  		fprintf(stderr, "factor was 0, using %d\n", currwin.shrinkfactor); */
/*  	    } */

	    /* else, try to find a suitable shrink factor: */
	    currwin.shrinkfactor = ROUNDUP(unshrunk_page_w, reply.width - 2);
/*  	    fprintf(stderr, "factor w: %d\n", currwin.shrinkfactor); */
	    
	    height_factor = ROUNDUP(unshrunk_page_h, reply.height - 2);
/*  	    fprintf(stderr, "factor h: %d\n", height_factor); */
	    if (height_factor >= currwin.shrinkfactor)
		currwin.shrinkfactor = height_factor;

/*  	    fprintf(stderr, "factor now is: %d\n", currwin.shrinkfactor); */
	    /* save current shrinkfactor for later use */
	    if (currwin.shrinkfactor > 1)
		bak_shrink = currwin.shrinkfactor;
	    
	    mane.shrinkfactor = currwin.shrinkfactor;
	    init_page();
	    set_wh_args[0].value = (XtArgVal) page_w;
	    set_wh_args[1].value = (XtArgVal) page_h;
	    *ret_w = page_w;
	    *ret_h = page_h;
	    XtSetValues(draw_widget, set_wh_args, XtNumber(set_wh_args));
	}
#ifdef MOTIF
	/*
	  SU 2002/11/23: Added scrollbar width to height computation.
	  Otherwise, when the vertical space isn't sufficient, a
	  vertical scrollbar will be added, but since this makes the
	  display narrower, a horizontal scrollbar will be added as
	  well, even if this wouldn't be neccessary.

	  SU 2003/09/30: Apparently the size computation works now, even
	  though I'm not sure why (scrollbar value isn't in add_w). Investigate.
	*/
	/* HACK ALERT: 4 for window decoration borders - FIXME: get actual values?? */
	set_wh_args[0].value = reply.width + 4 + add_w;
	set_wh_args[1].value = reply.height - 4 + add_h;
	XtSetValues(top_level, set_wh_args, XtNumber(set_wh_args));

#else /* MOTIF */

	set_wh_args[0].value = reply.width + add_w + 15;
	/*
	  FIXME: use real height of statusline here
	  Somehow I didn't manage to use XtVaCreateWidget in a call to
	  create_statusline() above, and XtManageChild() below.
	  In that case, we could do without global_statusline_h.
	 */
	set_wh_args[1].value = reply.height;
	XtSetValues(top_level, set_wh_args, XtNumber(set_wh_args));
	set_wh_args[0].value -= add_w;
	XtSetValues(vport_widget, set_wh_args, XtNumber(set_wh_args));

#endif /* MOTIF */
	*ret_w = set_wh_args[0].value;
	*ret_h = set_wh_args[1].value;
    }
}

static void
net_wm_toggle_fullscreen(int flag)
{
    Atom NET_WM_FULLSCREEN = XInternAtom(DISP, "_NET_WM_STATE_FULLSCREEN", True);

    if (NET_WM_FULLSCREEN) {
	XEvent ev;
	Atom NET_WM_STATE = XInternAtom(DISP, "_NET_WM_STATE", False);

	XDVI_INFO((stdout, "trying _NET_WM_STATE_FULLSCREEN ..."));

	ev.type = ClientMessage;
	ev.xclient.serial = 0;
	ev.xclient.send_event = True;
	ev.xclient.display = DISP;
	ev.xclient.window = XtWindow(top_level);
	ev.xclient.message_type = NET_WM_STATE;
	ev.xclient.format = 32;
	ev.xclient.data.l[0] = flag; /* _NET_WM_STATE_REMOVE (0) or _NET_WM_STATE_ADD (1) */
	ev.xclient.data.l[1] = NET_WM_FULLSCREEN;
	ev.xclient.data.l[2] = 0L;
	
	XSendEvent(DISP, DefaultRootWindow(DISP), False,
		   SubstructureNotifyMask, &ev);
    }
    else {
	XDVI_INFO((stdout, "_NET_WM_STATE_FULLSCREEN not supported by this window manager."));
    }
}

void
reconfigure_window(Boolean fullscreen, Dimension width, Dimension height,
		    Boolean save_position)
{
    static int x_old = 5, y_old = 15;
#ifdef SIZECONFIGURE_WORKS
    int sizeconfiguremask;
    XWindowChanges sizeconfigure;
#endif /* SIZECONFIGURE_WORKS */
#ifdef MOTIF
    static int save_wm_decorations;
#else
    static PropMotifWmHints MWMHints = {MWM_HINTS_DECORATIONS, 0, 0, 0, 0};
    Atom WM_HINTS = XInternAtom(DISP, "_MOTIF_WM_HINTS", True);
#endif
    int x, y;

#ifdef SIZECONFIGURE_WORKS
    sizeconfiguremask = CWX | CWY | CWWidth | CWHeight | CWBorderWidth;
    sizeconfigure.width = width;
    sizeconfigure.height = height;
#endif /* SIZECONFIGURE_WORKS */
    
    if (fullscreen) {
#if 0
	static Boolean first_time = True;
	/* offsets between the raw window and the decorated window */
	static int wm_x_offset = 0;
	static int wm_y_offset = 0;
#endif
	Window dummy;

#ifdef SIZECONFIGURE_WORKS
	sizeconfigure.x = 0;
	sizeconfigure.y = 0;
	sizeconfigure.border_width = 0;
#endif /* SIZECONFIGURE_WORKS */
	
	/* Note ZLB: avoid to call XTranslateCoordinates if the window is
	   not yet mapped since XTranslateCoordinates implicitly maps the
	   window (note: calling the function XGetWindowAttributes also
	   makes the window mapped).

	   This effectively eliminates the flashing effect when xdvik is
	   started in fullscreen mode for the Motif version. For the Xaw
	   version we still get flashing effect (seems that the window is
	   already mapped). */

        if (save_position) {
	   /* save current window coordinates so that we can change them back */
	    (void)XTranslateCoordinates(DISP, XtWindow(top_level),
					RootWindowOfScreen(SCRN),
					0, 0,
					&x_old, &y_old,
					&dummy);
#if 0
            if (first_time) {
		first_time = False;
		/* This is a hack for finding out wm_x_offset, wm_y_offset */
		XMoveWindow(DISP, XtWindow(top_level), 0, 0);
		XSync(DISP, False);
		(void)XTranslateCoordinates(DISP, XtWindow(top_level),
					RootWindowOfScreen(SCRN),
					0, 0, &wm_x_offset, &wm_y_offset,
					&dummy);
		fprintf(stderr, "wm offset = (%d,%d)\n",
			wm_x_offset, wm_y_offset);
	    }
	    x_old -= wm_x_offset;
	    y_old -= wm_y_offset;
#endif
        }

	x = y = 0;

#ifdef MOTIF
	XtVaGetValues(top_level, XmNmwmDecorations, &save_wm_decorations, NULL);
	/* FIXME: this doesn't work e.g. with KDE */
	XtVaSetValues(top_level,
		      XmNmwmDecorations, 0,
		      XmNgeometry, "+0+0",
		      NULL);
	XtVaSetValues(main_window, XmNshadowThickness, 0, NULL);
#else
	MWMHints.decorations = 0;
	if (WM_HINTS != None) {
	    XChangeProperty(DISP, XtWindow(top_level),
			    WM_HINTS, WM_HINTS, 32,
			    PropModeReplace, (unsigned char *)&MWMHints,
			    sizeof(MWMHints) / 4);
	}
#endif
	/* SU: apparently some new standard that is supposed to work
	   with some WMs ... but it doesn't with my versions of Gnome and KDE. */
	net_wm_toggle_fullscreen(1);

#ifdef SIZECONFIGURE_WORKS
	XConfigureWindow(DISP, XtWindow(top_level), sizeconfiguremask, &sizeconfigure);
#endif /* SIZECONFIGURE_WORKS */
    }
    else {
#ifdef SIZECONFIGURE_WORKS
	sizeconfiguremask = CWWidth | CWHeight | CWX | CWY;
	sizeconfigure.x = x_old;
	sizeconfigure.y = y_old;
	sizeconfigure.border_width = 20;
#endif /* SIZECONFIGURE_WORKS */
	XtVaSetValues(top_level, XtNx, x_old, XtNy, y_old, NULL);
#ifdef MOTIF
	XtVaSetValues(top_level, XmNmwmDecorations, save_wm_decorations, NULL);
	XtVaSetValues(main_window, XmNshadowThickness, g_save_shadow_thickness, NULL);
#else
	MWMHints.decorations = MWM_DECOR_ALL;
	if (WM_HINTS != None) {
	    XChangeProperty(DISP, XtWindow(top_level),
			    WM_HINTS, WM_HINTS, 32,
			    PropModeReplace, (unsigned char *)&MWMHints,
			    sizeof(MWMHints) / 4);
	}
#endif
	net_wm_toggle_fullscreen(0);

	x = x_old;
	y = y_old;
    }

#if 1
#if 0
    XUnmapWindow(DISP, XtWindow(top_level));
#else
    XWithdrawWindow(DISP, XtWindow(top_level), XScreenNumberOfScreen(SCRN));
#endif

    /* Note ZLB: Placing XResizeWindow before XUnmapWindow or after XMapWindow
	makes the fullscreen window size smaller than the screen size when
	using `mwm' of Lesstif */
    XSetWindowBorderWidth(DISP, XtWindow(top_level), 0);
    XResizeWindow(DISP, XtWindow(top_level), width, height);

    XMapRaised(DISP, XtWindow(top_level));

    /* Note ZLB: XMapWindow might change the window position with some WMs
	(like Sawfish), so we place the window position after it's mapped. */
    XMoveWindow(DISP, XtWindow(top_level), x, y);
#endif /* 0 */

    /* need to redraw the page to avoid artifacts */
    ev_flags |= EV_NEWPAGE;
    XFlush(DISP);
}

/*
  a custom error handler that makes it easier to catch X errors in the debugger
  (just set a breakpoint to this function, but don't forget the -sync option)
*/ 
static int
x_error_handler(Display *display, XErrorEvent *error)
{
    char buf[1024];
    XGetErrorText(display, error->error_code, buf, sizeof buf);
    XtCloseDisplay(DISP);
    XDVI_ABORT((stderr, "Caught X protocol error: %s on protocol request %d.\n"
		"I'm aborting now, to help you debugging this.",
		buf, error->request_code));
    /* notreached */
    return 0;
}


#ifdef HTEX
/*
 * Parse colors from resource.{visited_}link_color, saving them
 * to g_{visited_}link_color and {visited_}link_pix.
*/
static void
get_link_colors(Pixel *link_pix, Pixel *visited_link_pix)
{
    XrmValue from1, from2, to1, to2;
	
    XColor exact, approx;
    double r, g, b;
    double factor = 65535.0;
    int ret;

    /* get rgb values from color for links */
    if ((ret = XLookupColor(DISP, our_colormap,
			    resource.link_color,
			    &exact, &approx)) != 0) {
    }
    else {
	XDVI_WARNING((stderr, "XLookupColor failed for resource.link_color \"%s\"\n"
		      "- using fallback color \"%s\".",
		      resource.visited_link_color, LINK_COLOR_FALLBACK));
	XLookupColor(DISP, our_colormap, LINK_COLOR_FALLBACK, &exact, &approx);
    }
	
#if 0
    fprintf(stderr, "lookup color for %s returned: %d, %d, %d\n",
	    resource.link_color, exact.red, exact.green, exact.blue);
#endif
    r = exact.red / factor;
    g = exact.green / factor;
    b = exact.blue / factor;
    g_link_color_rgb = xmalloc(strlen("push rgb 0.00 0.00 0.00") + 1);
    sprintf(g_link_color_rgb, "push rgb %.2f %.2f %.2f", r, g, b);
	
    /* same for visited links */
    if ((ret = XLookupColor(DISP, our_colormap,
			    resource.visited_link_color,
			    &exact, &approx)) != 0) {
    }
    else {
	XDVI_WARNING((stderr, "XLookupColor failed for resource.visited_link_color \"%s\"\n"
		      "- using fallback color \"%s\".",
		      resource.visited_link_color, VISITED_LINK_COLOR_FALLBACK));
	XLookupColor(DISP, our_colormap, VISITED_LINK_COLOR_FALLBACK, &exact, &approx);
    }
	
#if 0
    fprintf(stderr, "lookup color for %s returned: %d, %d, %d\n",
	    resource.visited_link_color, exact.red, exact.green, exact.blue);
#endif
    r = exact.red / factor;
    g = exact.green / factor;
    b = exact.blue / factor;
    g_visited_link_color_rgb = xmalloc(strlen("push rgb 0.00 0.00 0.00") + 1);
    sprintf(g_visited_link_color_rgb, "push rgb %.2f %.2f %.2f", r, g, b);

    /* 2nd part: Create CG for the underlines. */
    from1.addr = resource.link_color;
    from1.size = strlen(from1.addr) + 1;
    to1.addr = (XtPointer)link_pix;
    to1.size = sizeof(Pixel);
    if (!XtConvertAndStore(top_level, XtRString, &from1, XtRPixel, &to1)) {
	XDVI_WARNING((stderr, "String to pixel conversion failed for resource.link_color \"%s\"\n"
		      "- using fallback color \"%s\".",
		      resource.link_color, LINK_COLOR_FALLBACK));
	from1.addr = LINK_COLOR_FALLBACK;
	from1.size = strlen(from1.addr) + 1;
	to1.addr = (XtPointer)link_pix;
	to1.size = sizeof(Pixel);
	XtConvertAndStore(top_level, XtRString, &from1, XtRPixel, &to1);
    }
    from2.addr = resource.visited_link_color;
    from2.size = strlen(from2.addr) + 1;
    to2.addr = (XtPointer)visited_link_pix;
    to2.size = sizeof(Pixel);
    if (!XtConvertAndStore(top_level, XtRString, &from2, XtRPixel, &to2)) {
	XDVI_WARNING((stderr, "String to pixel conversion failed for resource.visited_link_color \"%s\"\n"
		      "- using fallback color \"%s\".",
		      resource.visited_link_color, VISITED_LINK_COLOR_FALLBACK));
	from2.addr = VISITED_LINK_COLOR_FALLBACK;
	from2.size = strlen(from2.addr) + 1;
	to2.addr = (XtPointer)visited_link_pix;
	to2.size = sizeof(Pixel);
	XtConvertAndStore(top_level, XtRString, &from2, XtRPixel, &to2);
    }
}
#endif

Widget main_row, menu_bar, form;


#ifdef MOTIF
/* make sure the translations for the drawing area are properly set
   (Motif 2.x seems to somehow overwrite them somewhere in the
   initialization phase; bug #610206).
*/
void
motif_translations_kludge(void)
{
    ASSERT(clip_widget != NULL, "clip_widget must have been initialized");
    ASSERT(draw_widget != NULL, "draw_widget must have been initialized");
    XtOverrideTranslations(clip_widget, XtParseTranslationTable(base_key_translations));
    XtOverrideTranslations(draw_widget, XtParseTranslationTable(base_key_translations));
}
#endif /* MOTIF */


/*
  Initialize internal data (most of them global ...) according to the values
  of resources/command-line arguments, warning user about illegal values
  etc.
*/
static void
init_check_resources(void)
{
    int i;
    Boolean matched;
    
    if (resource.version_flag) {
	display_version_info();
	exit(EXIT_SUCCESS);
    }

    if (resource.mfmode != NULL) {
	char *p;

	p = strrchr(resource.mfmode, ':');
	if (p != NULL) {
	    unsigned int len;
	    char *p1;

	    ++p;
	    len = p - resource.mfmode;
	    p1 = xmalloc(len);
	    bcopy(resource.mfmode, p1, len - 1);
	    p1[len - 1] = '\0';
	    resource.mfmode = p1;
	    resource.pixels_per_inch = atoi(p);
	}
    }
    if (currwin.shrinkfactor < 0) {
	XDVI_ERROR((stderr, "Invalid shrink factor: %d.", currwin.shrinkfactor));
	usage(EXIT_FAILURE);
    }
    if (resource.density <= 0) {
	XDVI_ERROR((stderr, "Invalid shrink density: %d.", resource.density));
	usage(EXIT_FAILURE);
    }
    if (resource.pixels_per_inch <= 0) {
	XDVI_ERROR((stderr, "Invalid dpi value: %d.", resource.pixels_per_inch));
	usage(EXIT_FAILURE);
    }

#ifdef HTEX
    if (resource.link_style < 0 || resource.link_style > 3) {
	XDVI_ERROR((stderr, "Unrecognized value %d for resource \"linkstyle\" (valid range is 0 - 3); assuming 3.",
		    resource.link_style));
	resource.link_style = 3;
    }
#endif /* HTEX */
    
    if (currwin.shrinkfactor > 1) {
	bak_shrink = currwin.shrinkfactor;
	mane.shrinkfactor = currwin.shrinkfactor;	/* otherwise it's 1 */
    }

    /* subpixel rendering */
    g_subpixel_order = SUBPIXEL_NONE;
    if (resource.sub_pixels != NULL) {
	if (memicmp(resource.sub_pixels, "rgb", 4) == 0)
	    g_subpixel_order = SUBPIXEL_RGB;
	else if (memicmp(resource.sub_pixels, "bgr", 4) == 0)
	    g_subpixel_order = SUBPIXEL_BGR;
	else {
	    XDVI_ERROR((stderr,
			"Unrecognized value \"%s\" for resource subpixels\n"
			"(possible values are: \"rgb\" or \"bgr\").",
			resource.sub_pixels));
	    xdvi_exit(EXIT_FAILURE);
	}
    }

    /* margins */
    if (resource.sidemargin)
	home_x = atopix(resource.sidemargin, False);
    if (resource.topmargin)
	home_y = atopix(resource.topmargin, False);
    offset_x = resource.xoffset ? atopix(resource.xoffset, True)
	: resource.pixels_per_inch;
    offset_y = resource.yoffset ? atopix(resource.yoffset, True)
	: resource.pixels_per_inch;

    /* text_encoding */
    matched = False;
    for (i = 0; encoding_mapping[i].encoding_str != NULL; i++) {
	if (memicmp(resource.text_encoding,
		    encoding_mapping[i].encoding_str,
		    strlen(encoding_mapping[i].encoding_str) + 1) == 0) {
	    resource.internal_encoding = encoding_mapping[i].encoding;
	    matched = True;
	    break;
	}
    }
    if (!matched) {
	resource.internal_encoding = encoding_mapping[0].encoding;
	XDVI_WARNING((stderr, "Unrecognized character set: \"%s\"; assuming \"%s\".\n"
		      "Valid names (case is not significant) are:",
		      resource.text_encoding, encoding_mapping[0].encoding_str));
	for (i = 0; encoding_mapping[i].encoding_str != NULL; i++) {
	    fprintf(stderr, "%s ", encoding_mapping[i].encoding_str);
	}
	fputc('\n', stderr);
    }
    
    /* paper type */
    if (!set_paper_type(resource.paper)) {
	_Xconst char **p;
	XDVI_ERROR((stderr, "Unrecognized paper type \"%s\". Possible values are:", resource.paper));
	for (p = paper_types; p < paper_types + XtNumber(paper_types); p += 2)
	    fprintf(stderr, "%s ", *p);
	fprintf(stderr,
		"\n(the names ending with `r' are `rotated' or `landscape' variants),\n"
		"or a specification `WIDTHxHEIGHT' followed by a dimension unit\n"
		"(pt, pc, in, bp, cm, mm, dd, cc, or sp).\n\n");
	xdvi_exit(EXIT_FAILURE);
    }

    /* magnifier sizes */
    for (i = 0; i < 5; ++i) {
	if (resource.mg_arg[i] != NULL) {
	    char *s;

	    mg_size[i].w = mg_size[i].h = atoi(resource.mg_arg[i]);
	    s = strchr(resource.mg_arg[i], 'x');
	    if (s != NULL) {
		mg_size[i].h = atoi(s + 1);
		if (mg_size[i].h <= 0)
		    mg_size[i].w = 0;
	    }
	}
    }

#ifdef PS
    if (resource.safer) {
	resource.allow_shell = False;
# ifdef PS_GS
	resource.gs_safer = True;
# endif /* PS_GS */
    }
# ifdef	PS_GS
    {
	_Xconst char *CGMcgm = "CGMcgm";
	_Xconst char *cgmp;

	cgmp = strchr(CGMcgm, resource.gs_palette[0]);
	if (cgmp == NULL)
	    XDVI_FATAL((stderr, "Invalid value %s for gs palette option", resource.gs_palette));
	if (cgmp >= CGMcgm + 3) {
	    static char gsp[] = "x";

	    gsp[0] = *(cgmp - 3);
	    resource.gs_palette = gsp;
	}
    }
# endif /* PS_GS */
#endif /* PS */

    /* Initialize expert_flag according to expert mode and related resources
       The old `-expert' flag overrides resource.expert_mode, `+expert' (or not
       setting it) just uses expert_mode.
     */
    if (resource.expert)
	expert_flag = XPRT_SHOW_NONE;
    else
	expert_flag = resource.expert_mode;
/*      fprintf(stderr, "++++++++ initializing expert_flag: %d\n", expert_flag); */
    check_expert_mode();
}

static void
create_cursors(void)
{
    XColor bg_Color;
    Pixmap temp;
    Pixmap mask;

    /* first try colors by name, as fix for #804294; fall back on
       WhitePixelOfScreen() etc. if this fails:
    */
    if (XParseColor(DISP, our_colormap, "black", &cr_Color) == 0) { /* lookup failure */
	bg_Color.pixel = WhitePixelOfScreen(SCRN);
	XQueryColor(DISP, our_colormap, &bg_Color);
    }
    if (XParseColor(DISP, our_colormap, "white", &bg_Color) == 0) { /* lookup failure */
	cr_Color.pixel = BlackPixelOfScreen(SCRN);
	XQueryColor(DISP, our_colormap, &cr_Color);
    }

    /* wait cursor */
#if 0 /* this makes no big difference for #804294 */
    temp = XCreatePixmapFromBitmapData(DISP, RootWindowOfScreen(SCRN),
				       (char *)time16_bits,
				       time16_width, time16_height,
				       1, 0, 1);
    mask = XCreatePixmapFromBitmapData(DISP, RootWindowOfScreen(SCRN),
				       (char *)time16_mask_bits,
				       time16_mask_width, time16_mask_height,
				       1, 0, 1);
#else
    temp = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)time16_bits,
				 time16_width, time16_height);
    mask = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)time16_mask_bits,
				 time16_mask_width, time16_mask_height);
#endif
    wait_cursor = XCreatePixmapCursor(DISP, temp, mask,
				      &cr_Color, &bg_Color,
				      time16_width / 2, time16_height / 2);
    XFreePixmap(DISP, temp);
    XFreePixmap(DISP, mask);

    /* ready cursor, magnifier by default */
    temp = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)magglass_bits,
				 magglass_width, magglass_height);
    mask = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)magglass_mask_bits,
				 magglass_mask_width, magglass_mask_height);

    ready_cursor = XCreatePixmapCursor(DISP, temp, mask,
				       &cr_Color, &bg_Color,
				       magglass_x_hot, magglass_y_hot);
    XFreePixmap(DISP, temp);
    XFreePixmap(DISP, mask);

    corrupted_cursor = XCreateFontCursor(DISP, XC_watch);

#if !COLOR
    XRecolorCursor(DISP, ready_cursor, &cr_Color, &bg_Color);
    XRecolorCursor(DISP, wait_cursor, &cr_Color, &bg_Color);
#endif
    /* Cursor when page is paused */
#ifdef VMS
    temp = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)hand_bits, hand_width, hand_height);
    mask = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)hand_mask_bits, hand_mask_width, hand_mask_height);
    pause_cursor = XCreatePixmapCursor(DISP, temp, mask,
				       &cr_Color, &bg_Color, 6, 6);
    XFreePixmap(DISP, temp);
    XFreePixmap(DISP, mask);
#else
    pause_cursor = XCreateFontCursor(DISP, XC_watch);
#endif

    /* cursors indicating dragging direction */
    temp = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)drag_vert_bits,
				 drag_vert_width, drag_vert_height);
    mask = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)drag_vert_mask_bits,
				 drag_vert_mask_width, drag_vert_mask_height);
    drag_cursor_v = XCreatePixmapCursor(DISP, temp, mask,
					&cr_Color, &bg_Color,
					drag_vert_x_hot, drag_vert_y_hot);
    XFreePixmap(DISP, temp);
    XFreePixmap(DISP, mask);

    temp = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)drag_horiz_bits,
				 drag_horiz_width, drag_horiz_height);
    mask = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)drag_horiz_mask_bits,
				 drag_horiz_width, drag_horiz_height);
    drag_cursor_h = XCreatePixmapCursor(DISP, temp, mask,
					&cr_Color, &bg_Color,
					drag_horiz_x_hot, drag_horiz_y_hot);
    XFreePixmap(DISP, temp);
    XFreePixmap(DISP, mask);

    temp = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)drag_omni_bits,
				 drag_omni_width, drag_omni_height);
    mask = XCreateBitmapFromData(DISP, RootWindowOfScreen(SCRN),
				 (char *)drag_omni_mask_bits,
				 drag_omni_mask_width, drag_omni_mask_height);
    drag_cursor_a = XCreatePixmapCursor(DISP, temp, mask,
				       &cr_Color, &bg_Color,
				       drag_omni_x_hot, drag_omni_y_hot);
    XFreePixmap(DISP, temp);
    XFreePixmap(DISP, mask);

#ifndef	VMS
    link_cursor = XCreateFontCursor(DISP, XC_hand2);
    rule_cursor = XCreateFontCursor(DISP, XC_crosshair);
    mag_cursor = XCreateFontCursor(DISP, XC_tcross);
#else
    link_cursor = ready_cursor;
    rule_cursor = ready_cursor;
    mag_cursor = ready_cursor;
#endif
}

/* Initialize a forward search if the source_position argument is not NULL. */
static void
do_forward_search(const char *source_position)
{
    /* parse the special to expand the filename */
    struct src_parsed_special data;
    char *new_special = NULL;
    char *expanded_filename = NULL;
	    
    if (source_position == NULL) /* nothing to do */
	return;
    
    data.filename_len = 0;
    data.filename = NULL;

    src_parse(source_position, &data);
	    
    if (data.filename_len == 0) { /* malformed argument: tell user, and ignore it */
	popup_message(MSG_ERR,
			 /* helptext */
			 "The format of the -sourceposition argument should be:\n"
			 "-sourceposition '<nn>[ ]*<filename'\n"
			 "with <nn> = linenumber, [ ]* = an arbitrary number of spaces, "
			 "and <filename> = TeX source file.",
			 /* error message */
			 "Filename missing in `-sourceposition' argument \"%s\". "
			 "Could not perform forward search.",
			 source_position);
    }
    else {
	TRACE_CLIENT((stderr, "got data: line %d, col %d, file |%s|, len %d\n",
		      data.line, data.col, data.filename, data.filename_len));
		
	/* expand -sourceposition argument if it contains a path component, like above */
	if (strchr(data.filename, '/') != NULL
	    && (expanded_filename = expand_filename(data.filename, USE_CWD_PATH)) != NULL) {
	    char *tmp = canonicalize_path(expanded_filename);
	    free(data.filename);
	    free(expanded_filename);
	    expanded_filename = tmp;		
	}
	else
	    expanded_filename = data.filename;
		
	TRACE_CLIENT((stderr, "expanded source_position: |%s|\n", expanded_filename));
		
	new_special = xmalloc(2 * LENGTH_OF_INT + 2 /* 2 for `:' and separating space */
			      + strlen(expanded_filename) + 1);
	sprintf(new_special, "%d:%d %s", data.line, data.col, expanded_filename);
	free(expanded_filename);
	source_forward_string = new_special;
	ev_flags |= EV_SRC;
    }
}

static void
check_early_arguments(int argc, char **argv)
{
    int i;
    /* This has to be a special case, for now. We need to loop through all args
     * in case xdvi is aliased, or a shell script (like in teTeX) that adds
     * things like `-name' at the beginning of the arglist.
     */
    for (i = 1; i < argc; i++) {
	if (strcmp(argv[i], "-help") == 0
	    || strcmp(argv[i], "-h") == 0
	    || strcmp(argv[i], "+help") == 0
	    || strcmp(argv[i], "--help") == 0) {
	    /*	extern char *kpse_bug_address; */
	    display_version_info();
	    printf("A DVI file previewer for the X window system.\n\n");
	    display_bug_reporting_info();
	    usage(0);
	}
	else if (strcmp(argv[i], "--version") == 0
		 || strcmp(argv[i], "-v") == 0) {
	    display_version_info();
	    display_licensing_info();
	    display_bug_reporting_info();
	    exit(EXIT_SUCCESS);
	}
	else if (strcmp(argv[i], "-sync") == 0) {
	    XDVI_INFO((stdout, "Found sync option; installing custom error handler \"x_error_handler\""));
	    /* in this case, we'll also want to install our custom error handler so
	       that X errors can be caught easier in the debugger:
	    */
	    (void)XSetErrorHandler(x_error_handler);
	}
    }
}

static void
create_colormaps(void)
{
#ifdef GREY
    our_depth = (unsigned int)DefaultDepthOfScreen(SCRN);
    our_visual = DefaultVisualOfScreen(SCRN);
    our_colormap = DefaultColormapOfScreen(SCRN);
#ifdef XSERVER_INFO
    if (debug & DBG_ALL)
	fprintf(stdout, "--- our_depth: %d\n", our_depth);
#endif

    if (resource.install != False && our_visual->class == PseudoColor) {
	/* look for a TrueColor visual with more bits */
	XVisualInfo template;
	XVisualInfo *list;
	int nitems_return;
#ifdef XSERVER_INFO
	if (debug & DBG_ALL)
	    fprintf(stdout, "--- looking for a better TrueColor visual\n");
#endif

	template.screen = XScreenNumberOfScreen(SCRN);
	template.class = TrueColor;
	list = XGetVisualInfo(DISP, VisualScreenMask | VisualClassMask,
			      &template, &nitems_return);
	if (list != NULL) {
	    XVisualInfo *list1;
	    XVisualInfo *best = NULL;

	    for (list1 = list; list1 < list + nitems_return; ++list1) {
#ifdef XSERVER_INFO
		if (debug & DBG_ALL)
		    fprintf(stdout, "--- checking %d\n", list1->depth);
#endif
		if ((unsigned int)list1->depth > our_depth
# if PS_GS
		    /* patch by Toni Ronkko <tronkko@hytti.uku.fi>, fixes bug #458057:
		     * SGI supports additional depths of 12 and 30, but
		     * these are not supported by ghostscript (see
		     * xdev->vinfo->depth in gdevxcmp.c (ghostscript-6.51)),
		     * so we restrict the values to those supported by gs.
		     */
		    && (list1->depth == 1 || list1->depth == 2
			|| list1->depth == 4 || list1->depth == 8
			|| list1->depth == 15 || list1->depth == 16
			|| list1->depth == 24 || list1->depth == 32)
# endif
		    && (best == NULL || list1->depth > best->depth))
		    best = list1;
	    }
	    if (best != NULL) {
#ifdef XSERVER_INFO
		if (debug & DBG_ALL)
		    fprintf(stdout, "--- best depth: %d\n", best->depth);
#endif
		our_depth = best->depth;
		our_visual = best->visual;
		our_colormap = XCreateColormap(DISP,
					       RootWindowOfScreen(SCRN),
					       our_visual, AllocNone);
		XInstallColormap(DISP, our_colormap);
		temp_args1[0].value = (XtArgVal) our_depth;
		temp_args1[1].value = (XtArgVal) our_visual;
		temp_args1[2].value = (XtArgVal) our_colormap;
		XtSetValues(top_level, temp_args1, XtNumber(temp_args1));
		XtSetTypeConverter(XtRString, XtRPixel,
				   XdviCvtStringToPixel,
				   (XtConvertArgList) colorConvertArgs, 2,
				   XtCacheByDisplay, NULL);
	    }
	    XFree(list);
	}
    }

#ifdef MOTIF
    if (debug & DBG_GUI) {
	printf("Compiled with %s, runtime version %d.%d\n",
	       /* 	   XmVERSION, XmREVISION, XmUPDATE_LEVEL, */
	       XmVERSION_STRING,
	       xmUseVersion / 1000, xmUseVersion % 1000);
    }
#endif
    
    if (resource.install == True && our_visual->class == PseudoColor) {
	XColor tmp_color;
#ifdef XSERVER_INFO
	if (debug & DBG_ALL)
	    fprintf(stdout, "--- PseudoColor, trying to install colormap\n");
#endif

	/* This next bit makes sure that the standard black and white pixels
	   are allocated in the new colormap. */
	tmp_color.pixel = BlackPixelOfScreen(SCRN);
	XQueryColor(DISP, our_colormap, &tmp_color);
	XAllocColor(DISP, our_colormap, &tmp_color);

	tmp_color.pixel = WhitePixelOfScreen(SCRN);
	XQueryColor(DISP, our_colormap, &tmp_color);
	XAllocColor(DISP, our_colormap, &tmp_color);

	our_colormap = XCopyColormapAndFree(DISP, our_colormap);
	temp_args1[2].value = (XtArgVal) our_colormap;
	XtSetValues(top_level, temp_args1 + 2, 1);
    }

    XtGetApplicationResources(top_level, (XtPointer) & resource,
			      app_pixel_resources,
			      XtNumber(app_pixel_resources), (ArgList) NULL, 0);

#endif /* GREY */

    copy = resource.copy;

#if GREY || COLOR
    fore_color_data.pixel = resource.fore_Pixel;
    back_color_data.pixel = resource.back_Pixel;
    XQueryColors(DISP, our_colormap, color_data, 2);
#endif
    
#if COLOR
    fg_initial.r = fore_color_data.red;
    fg_initial.g = fore_color_data.green;
    fg_initial.b = fore_color_data.blue;
    bg_initial.r = back_color_data.red;
    bg_initial.g = back_color_data.green;
    bg_initial.b = back_color_data.blue;
#endif

#if GREY
    if (our_depth == 1) {
#ifdef XSERVER_INFO
	if (debug & DBG_ALL)
	    fprintf(stdout, "--- using depth 1\n");
#endif
	resource.use_grey = False;
    }
    if (resource.use_grey && our_visual->class != TrueColor) {
#ifdef XSERVER_INFO
	if (debug & DBG_ALL)
	    fprintf(stdout, "--- using grey, but not TrueColor\n");
#endif
	init_plane_masks();
	if (!copy) {
	    /* Retain back_color_data.pixel for psgs.c.  */
	    XColor tmp_color;
#ifdef XSERVER_INFO
	    if (debug & DBG_ALL)
		fprintf(stdout, "--- not using copy\n");
#endif
	    
	    tmp_color = back_color_data;
	    tmp_color.pixel = resource.back_Pixel;
	    XStoreColor(DISP, our_colormap, &tmp_color);
	}
    }
#endif
}

static void
create_initialize_widgets(
#ifdef MOTIF
			  Widget tool_bar
#else
			  void
#endif
			  )
{
    Dimension main_win_w, main_win_h;
    int add_w, add_h;
    
    static _Xconst char default_key_translations[] =
	"\"0\":digit(0)\n"
	"\"1\":digit(1)\n"
	"\"2\":digit(2)\n"
	"\"3\":digit(3)\n"
	"\"4\":digit(4)\n"
	"\"5\":digit(5)\n"
	"\"6\":digit(6)\n"
	"\"7\":digit(7)\n"
	"\"8\":digit(8)\n"
	"\"9\":digit(9)\n"
	/* 		"\"-\":minus()\n" */
	"<Motion>:motion()\n";
    
    static _Xconst char default_mouse_translations[] =
	"<BtnUp>:release()";

#ifdef MOTIF
    Widget status_line;

#if 0
    /*
     * Try to determine width of longest page string:
     */
    for (i = 0; i < total_pages; i++) {
	int curr = abs(pageinfo_get_number(i));
	if (curr > longest_page)
	    longest_page = curr;
    }
    fprintf(stderr, "longest page number: %d\n", longest_page);
    sprintf(tmpbuf, "* %d", longest_page);
    tmpstring = XmStringCreateLocalized(tmpbuf);
    dummy_list = XtCreateWidget("PageList", xmListWidgetClass, form, NULL, 0);
    fprintf(stderr, "rendition table\n");
    n = 0;
    XtSetArg (args[n], XmNfontName, "-*-courier-*-r-*--*-120-*"); n++;
    XtSetArg (args[n], XmNfontType, XmFONT_IS_FONT); n++;
    rendition = XmRenditionCreate (dummy_list, XmFONTLIST_DEFAULT_TAG, args, n);
    rtable = XmRenderTableAddRenditions (NULL, &rendition, 1, XmREPLACE);
    XtVaSetValues(dummy_list, XmNrenderTable, rtable, NULL);
    
    /* 	rendition = XmRenditionCreate(top_level, XmFONTLIST_DEFAULT_TAG, NULL, 0); */
    fprintf(stderr, "extent\n");
    XmStringExtent(rendition, tmpstring, &width, &height);
    fprintf(stderr, "string %s has width %d, height %d\n", tmpbuf, width, height);
    XtDestroyWidget(dummy_list);
    XmRenditionFree(rendition);
    XmStringFree(tmpstring);
#endif
    {
#define ARG_LEN 20
	int n = 0;
	Arg list_args[ARG_LEN];

	XtSetArg(list_args[n], XmNlistSizePolicy, XmVARIABLE);			n++;
	XtSetArg(list_args[n], XmNwidth, resource.pagelist_width);		n++;
	XtSetArg(list_args[n], XmNheight, page_h);				n++;
/* 	XtSetArg(list_args[n], XmNspacing, 10);					n++; */
	XtSetArg(list_args[n], XmNtopAttachment, XmATTACH_FORM);		n++;
	XtSetArg(list_args[n], XmNbottomAttachment, XmATTACH_FORM);		n++;
	XtSetArg(list_args[n], XmNleftAttachment, XmATTACH_FORM);		n++;
	XtSetArg(list_args[n], XmNrightAttachment, XmATTACH_OPPOSITE_FORM);	n++;
	XtSetArg(list_args[n], XmNrightOffset, -resource.pagelist_width);	n++;
	XtSetArg(list_args[n], XmNresizable, True);				n++;
	XtSetArg(list_args[n], XmNtopOffset, 2);				n++;
	XtSetArg(list_args[n], XmNleftOffset, 2);				n++;
	ASSERT(n < ARG_LEN, "list_args too short");
#undef ARG_LEN
	/* TODO: Find a smart way to determine size of pagelist instead of using resource.pagelist_width,
	   and find a smart way of resizing it when switching to a file with larger page numbers!
	*/
	page_list = XmCreateScrolledList(form, "PageList", list_args, n);
    }
    
    if (expert_flag == XPRT_SHOW_NONE) {
	XtUnmanageChild(XtParent(page_list));
	XtUnmanageChild(page_list);
    }
    
    main_window = XtVaCreateManagedWidget("mainWindow",
					  xmScrolledWindowWidgetClass, form,
					  XmNscrollingPolicy, XmAUTOMATIC,
					  XmNleftAttachment, XmATTACH_WIDGET,
					  /* lesstif balks if we just use page_list, so use its parent,
					     the xmScrolledWindow, for alignment: */
					  XmNleftWidget, XtParent(page_list),
					  XmNallowResize, True,
/* 					  XmNtopOffset, 2, */
					  XmNtopAttachment, XmATTACH_FORM,
					  XmNbottomAttachment, XmATTACH_FORM,
					  XmNrightAttachment, XmATTACH_FORM,
					  /* 					      XmNleftOffset, 10, */
					  XmNresizable, True,
					  XmNborderWidth, 0,
					  NULL);
    XtManageChild(form);

    x_bar = XtNameToWidget(main_window, "HorScrollBar");
    y_bar = XtNameToWidget(main_window, "VertScrollBar");

    
#ifdef TEST_SCROLLING
    /* TODO: try the following to prevent `flashing' effect - still breaks
       scrolling in its current form though */
    draw_background = XtVaCreateManagedWidget("drawing_bg",
				       xmFormWidgetClass, main_window,
				       XmNwidth, page_w,
				       XmNheight, page_h,
				       XmNtopAttachment, XmATTACH_FORM,
				       XmNbottomAttachment, XmATTACH_FORM,
				       XmNrightAttachment, XmATTACH_FORM,
				       XmNleftAttachment, XmATTACH_FORM,
				       XmNhighlightThickness, 0,
				       XmNbackground, resource.back_Pixel,
				       XmNrubberPositioning, True,
				       NULL);
#endif
    draw_widget = XtVaCreateWidget("drawing",
				   xmDrawingAreaWidgetClass,
#ifdef TEST_SCROLLING
				   draw_background,
#else
				   main_window,
#endif
				   XmNwidth, page_w,
				   XmNheight, page_h,
				   XmNhighlightThickness, 0,
				   XmNrubberPositioning, True,
				   XtNbackground, resource.back_Pixel,
				   NULL);

#if !FIXED_FLUSHING_PAGING
    ASSERT(XtParent(draw_widget) != NULL, "");
    XtVaSetValues(XtParent(draw_widget), XtNbackground, resource.back_Pixel, NULL);
#endif
    
#ifdef TEST_SCROLLING
    fprintf(stderr, "draw_widget is: %p\n", draw_widget);
#endif
    XtVaGetValues(main_window, XmNclipWindow, &clip_widget, NULL);

    XtVaSetValues(main_row,
		  XmNmenuBar, menu_bar,
		  XmNcommandWindow, XtParent(tool_bar),
		  XmNworkWindow, form,
		  NULL);
    
    XtVaSetValues(main_window,
		  XmNworkWindow, draw_widget,
		  NULL);

    XtManageChild(draw_widget);

    status_line = create_statusline(main_row);
    XtVaSetValues(main_row,
		  XmNmessageWindow, status_line,
		  NULL);
    if ((expert_flag & XPRT_SHOW_STATUSLINE) == 0) {
	XtUnmanageChild(status_line);
    }

    XtManageChild(main_row);

    if ((expert_flag & XPRT_SHOW_TOOLBAR) != 0) {
	if (resource.toolbar_unusable)
	    statusline_print(STATUS_LONG,
			     "There has been an error creating the toolbar pixmaps - toolbar is disabled!");
	else
	    XtManageChild(XtParent(tool_bar));
    }

    if ((expert_flag & XPRT_SHOW_MENUBAR) != 0)
	XtManageChild(menu_bar);
    
    XmAddTabGroup(draw_widget);
    
    /*
      note: a few more custom translations for page_list are defined in
      pagesel.c, since the actions are only known there.
    */
    {
	size_t i, k = 0;
#define WIDGETS_SIZE 10
	Widget widgets[WIDGETS_SIZE];

	widgets[k++] = main_row;
/* 	widgets[k++] = menu_bar; */
	widgets[k++] = tool_bar;
	widgets[k++] = page_list;
	widgets[k++] = main_window;
	widgets[k++] = draw_widget;
	widgets[k++] = clip_widget;

	ASSERT(k < WIDGETS_SIZE, "widgets list too short");
#undef WIDGETS_SIZE
	for (i = 0; i < k; i++) {
	    XtOverrideTranslations(widgets[i], XtParseTranslationTable(base_key_translations));
	    XtOverrideTranslations(widgets[i], XtParseTranslationTable(default_key_translations));
	    if (i > 3) { /* widgets for which we want to use our own mouse translations */
		XtOverrideTranslations(widgets[i], XtParseTranslationTable(base_mouse_translations));
		XtOverrideTranslations(widgets[i], XtParseTranslationTable(default_mouse_translations));
	    }
	}
    }
    
    if (resource.main_translations != NULL) {
	XtOverrideTranslations(draw_widget, XtParseTranslationTable(resource.main_translations));
	XtOverrideTranslations(clip_widget, XtParseTranslationTable(resource.main_translations));
	/* don't do it for the page list, otherwise mouse customizations will break
	   the default list bindings too. */
	/* XtOverrideTranslations(page_list, XtParseTranslationTable(resource.main_translations)); */
    }
    /* widgets for which wheel mouse should scroll the drawing area */
    if (resource.wheel_unit != 0 && resource.wheel_translations != NULL) {
	XtOverrideTranslations(main_row, XtParseTranslationTable(resource.wheel_translations));
	XtOverrideTranslations(tool_bar, XtParseTranslationTable(resource.wheel_translations));
	XtOverrideTranslations(main_window, XtParseTranslationTable(resource.wheel_translations));
	XtOverrideTranslations(draw_widget, XtParseTranslationTable(resource.wheel_translations));
/*  	XtOverrideTranslations(draw_background, XtParseTranslationTable(resource.wheel_translations)); */
    }
    
#else /* MOTIF */
    form_widget = XtVaCreateManagedWidget("form", formWidgetClass, top_level,
					  XtNdefaultDistance, 0,
					  NULL);
    vport_widget = XtVaCreateManagedWidget("vport", viewportWidgetClass,
					   form_widget,
					   XtNborderWidth, 0,
					   XtNtop, XtChainTop,
					   XtNbottom, XtChainBottom,
					   XtNleft, XtChainLeft,
					   XtNright, XtChainRight,
					   XtNallowHoriz, True,
					   XtNallowVert, True,
					   NULL);

    clip_widget = XtNameToWidget(vport_widget, "clip");

    draw_widget = XtVaCreateManagedWidget("drawing", drawWidgetClass, vport_widget,
					  XtNwidth, page_w,
					  XtNheight, page_h,
					  XtNx, 0,
					  XtNy, 0,
					  XtNlabel, "",
					  NULL);

    XtOverrideTranslations(form_widget, XtParseTranslationTable(base_key_translations));
    XtOverrideTranslations(form_widget, XtParseTranslationTable(base_mouse_translations));
    XtOverrideTranslations(form_widget, XtParseTranslationTable(default_key_translations));
    XtOverrideTranslations(form_widget, XtParseTranslationTable(default_mouse_translations));

    if (resource.wheel_unit != 0 && resource.wheel_translations != NULL)
	XtOverrideTranslations(form_widget, XtParseTranslationTable(resource.wheel_translations));

    if (resource.main_translations != NULL) {
	XtOverrideTranslations(form_widget, XtParseTranslationTable(resource.main_translations));
    }
    
    
    /* set background colors of the drawing widget */
    XtVaSetValues(draw_widget, XtNbackground, resource.back_Pixel, NULL);
    
#if !FIXED_FLUSHING_PAGING
    XtVaSetValues(clip_widget, XtNbackground, resource.back_Pixel, NULL);
#endif

    if ((expert_flag & XPRT_SHOW_BUTTONS) != 0)
	create_menu_buttons(form_widget, &add_w);
#endif /* MOTIF */

    /* activate expert mode and related settings */
    toggle_statusline();
    toggle_scrollbars();
#ifdef MOTIF
    toggle_pagelist();
    toggle_toolbar();
    toggle_menubar();
#else
    toggle_buttons();
#endif

    /*
     *	Set initial window size.
     *	This needs to be done before colors are assigned because if
     *	-s 0 is specified, we need to compute the shrink factor
     *	(which in turn affects whether init_pix is called).
     */

#ifdef MOTIF
    add_w = 0;
    if (expert_flag & XPRT_SHOW_PAGELIST)
	add_w += xm_get_width(page_list);
/*      if (y_bar != NULL) */
/*  	add_w += xm_get_width(y_bar); */
    add_h = 0;
    if (expert_flag & XPRT_SHOW_MENUBAR)
	add_h += xm_get_height(menu_bar);
    if (expert_flag & XPRT_SHOW_TOOLBAR)
	add_h += xm_get_height(tool_bar);
    if (expert_flag & XPRT_SHOW_STATUSLINE)
	add_h += xm_get_height(status_line);
/*      if (x_bar != NULL) */
/*  	add_h += xm_get_width(x_bar); */
#else
    /* add_w has been initialized by create_menu_buttons() call above */
    add_h = 0;
#endif
    set_windowsize(&main_win_w, &main_win_h, add_w, add_h, False);
    
    /*
     *	Realize the widgets (or windows).
     */

#ifndef MOTIF
    if ((expert_flag & XPRT_SHOW_BUTTONS) != 0) {
	set_button_panel_height(main_win_h);
    }
#endif
    XtAddEventHandler(
#ifdef MOTIF
		      clip_widget,
#else
		      vport_widget,
#endif
		      StructureNotifyMask, False,
		      handle_resize, (XtPointer) NULL);
    
    XtAddEventHandler(top_level, PropertyChangeMask, False,
		      handle_property_change, (XtPointer) NULL);
    XtAddEventHandler(draw_widget, ExposureMask, False, handle_expose,
		      (XtPointer) &mane);
    XtRealizeWidget(top_level);

#ifdef MOTIF
    /* for Xaw, event handlers for scrollbars are added inside get_geom(), events.c */
    XtAddEventHandler(x_bar, ButtonPressMask | ButtonReleaseMask, False, handle_x_scroll, NULL);
    XtAddEventHandler(y_bar, ButtonPressMask | ButtonReleaseMask, False, handle_y_scroll, NULL);
#else
    mainDeleteWindow = XInternAtom(DISP, "WM_DELETE_WINDOW", False);
    XSetWMProtocols(DISP, XtWindow(top_level), &mainDeleteWindow, 1);
    XtAddEventHandler(top_level, NoEventMask, True, handle_delete_message, NULL);
#endif
    
    /* check whether we want to run in fullscreen mode */
    if (resource.fullscreen) {
	reconfigure_window(resource.fullscreen, main_win_w + add_w, main_win_h, False);
    }

#ifdef MOTIF
    XmProcessTraversal(draw_widget, XmTRAVERSE_CURRENT);
    TRACE_GUI((stderr, "toplevel: %p", (void *)top_level));
    create_tips(top_level);
#else
    if ((expert_flag & XPRT_SHOW_STATUSLINE) != 0) {
	create_statusline();
    }
#endif

    currwin.win = mane.win = XtWindow(draw_widget);

    {
	XWindowAttributes attrs;

	(void)XGetWindowAttributes(DISP, mane.win, &attrs);
	backing_store = attrs.backing_store;
    }

#ifdef HAVE_X11_XMU_EDITRES_H
    /*
     * Enable editres protocol (see "man editres").
     * Usually will need to add -lXmu to the linker line as well.
     */
    XtAddEventHandler(top_level, (EventMask)0, True, _XEditResCheckMessages,
	    (XtPointer)NULL);
#endif
}

/*
 * Create colors and GCs.
 * In color mode, color changes affect foreGC, foreGC2,
 * and ruleGC, but not copyGC or highGC.
 */
static void
create_gcs(void)
{
    
#if GREY
    if (resource.gamma == 0.0)
	resource.gamma = 1.0;
#endif

    if (!resource.rule_color)
	resource.rule_pixel = resource.fore_Pixel;

#if !COLOR

#if GREY
    if (resource.use_grey)
	init_pix();
    else
#endif
	{ /* not #defined GREY, or not resource.use_grey */
	    XGCValues values;
	    Pixel set_bits = (Pixel) (resource.fore_Pixel & ~resource.back_Pixel);
	    Pixel clr_bits = (Pixel) (resource.back_Pixel & ~resource.fore_Pixel);
	    Boolean copy_tmp = resource.copy;
	    
	    copyGC = make_gc(GXcopy, resource.fore_Pixel, resource.back_Pixel);
	    if (copy_tmp || (set_bits && clr_bits)) {
		ruleGC = copyGC;
		if (!resource.thorough)
		    copy_tmp = True;
	    }
	    if (copy_tmp) {
		foreGC = ruleGC;
		if (!resource.copy) {
		    warn_overstrike();
		}
	    }
	    else {
		if (set_bits) {
		    foreGC = make_gc(GXor, set_bits, 0);
		    invertedGC = make_gc(GXor, 0, set_bits);
		}
		if (clr_bits || !set_bits)
		    *(foreGC ? &foreGC2 : &foreGC) = make_gc(GXandInverted, clr_bits, 0);
		if (!ruleGC)
		    ruleGC = foreGC;
	    }
	}
#endif /* !COLOR */
    
#ifdef HTEX
    {
	Pixel link_pix, visited_link_pix;
	get_link_colors(&link_pix, &visited_link_pix);
	linkcolorGC = make_gc(GXcopy, link_pix, resource.back_Pixel);
	visited_linkcolorGC = make_gc(GXcopy, visited_link_pix, resource.back_Pixel);
    }
#endif /* HTEX */
	
#if COLOR
    /* Not affected by color changes.  */
    copyGC = make_gc(GXcopy, resource.fore_Pixel, resource.back_Pixel);
#endif
    highGC = make_gc(GXcopy, resource.hl_Pixel, resource.back_Pixel);
    invertedGC = make_gc(GXcopy, resource.back_Pixel, resource.fore_Pixel);
    
    rulerGC = make_gc(GXcopy, resource.rule_pixel, resource.fore_Pixel);
    
#if XAW
    /*
     * There's a bug in the Xaw toolkit, in which it uses the
     * DefaultGCOfScreen to do vertical scrolling in the Text widget.
     * This leads to a BadMatch error if our visual is not the default one.
     * The following kludge works around this.
     */
    DefaultGCOfScreen(SCRN) = copyGC;
#endif
}

static void
do_fork(void)
{
    TRACE_CLIENT((stderr, "no other instance of xdvi found, forking ..."));
    /*
     * No suitable xdvi found, so we start one by
     * self-backgrounding.
     */
    /* flush output buffers to avoid double buffering (i.e. data
       waiting in the output buffer being written twice, by the parent
       and the child) */
    fflush(stdout);
    fflush(stderr);
    XFlush(DISP);
    if (fork())	/* if initial process (do NOT use vfork()!) */
	_exit(0);
}

/*
  ************************************************************
  ************************************************************
			main routine
  ************************************************************
  ************************************************************
*/
int
main(int argc, char **argv)
{
    Boolean tried_dvi_ext = False;
#ifdef MOTIF
    Widget tool_bar = 0;
#endif

    char *title_name = NULL;
    char *icon_name = NULL;
    char *page_arg = NULL;
    dvi_err_flag_t errflag = NO_ERROR;
    int i;
    
#ifdef DEBUG
#endif /* DEBUG */

    warn_about_prerelease_versions();
/*     BEGIN_TIMER_LOOP; */
    
    /*
     *      Step 1:  Process command-line options and resources.
     */

#ifndef	VMS
    prog = strrchr(*argv, '/');
#else
    prog = strrchr(*argv, ']');
#endif
    if (prog != NULL)
	++prog;
    else
	prog = *argv;

#ifdef	VMS
    if (strchr(prog, '.') != NULL)
	*strchr(prog, '.') = '\0';
#endif

    if (strstr(XDVI_VERSION_INFO, "-cvs") != NULL
	|| strstr(XDVI_VERSION_INFO, "-beta") != NULL) {
	MYTRACE((stderr, "installing custom error handler \"x_error_handler\""));
	(void)XSetErrorHandler(x_error_handler);
    }

    /* arguments that need to be checked early, like `help', `version' and `sync'. */
    check_early_arguments(argc, argv);
    
    /* We need to set up SIGALRM before calling XtAppAddTimeOut() (inside sfSelFile, or
       called from XtInitialize() in Motif), else and we'll die with the error message
       `Alarm clock'. However, we mustn't install SIGPOLL before the forking is done
       below, else xdvi may hang forever waiting for input. So the signal handler setup
       is split in 2 parts: setup_sigalarm(), and setup_signal_handlers() below.
    */
    setup_sigalarm();

    /* get the debug value (if any) from the environment */
    /* (it's too early to get it from the command line) */
    debug = parse_debugging_option(getenv("XDVIDEBUG"));

    top_level = XtInitialize(prog, "XDvi", options, XtNumber(options),
			     &argc, argv);

    app = XtWidgetToApplicationContext(top_level);
    
    XtAppAddActions(app, actions, num_actions);
    create_magnifier();

    /* at this point, all known options will have been consumed; so the
       only ones remaining should be: +pageno, and the dvi name. Exit with
       usage message if this is not the case. The following rather complicated
       looping logic ensures that the first unknown option is regarded as dvi_name,
       unless it's followed by another option, in which case it will result in an
       `Unrecognized option' error.
    */
    for (i = 0; i < argc - 1; i++) { /* argc - 1 because we're going to look at
					argv[i + 1] to skip argv[0] */
	if (*(argv[i + 1]) == '+') {
	    if (page_arg != NULL) {
		XDVI_ERROR((stderr, "Unrecognized option `%s'.", argv[i + 1]));
		usage(EXIT_FAILURE);
	    }
	    else {
		page_arg = argv[i + 1] + 1;
	    }
	}
	else if (dvi_name == NULL) {
	    dvi_name = xstrdup(argv[i + 1]);
	}
	else {
	    XDVI_ERROR((stderr, "Unrecognized option `%s'.", argv[i]));
	    usage(EXIT_FAILURE);
	}
    }

    DISP = XtDisplay(top_level);
    SCRN = XtScreen(top_level);

#ifdef GREY
    XtSetTypeConverter(XtRString, XtRBool3, XdviCvtStringToBool3,
		       NULL, 0, XtCacheNone, NULL);
#endif

#ifndef MOTIF
    accels_cr = XtParseAcceleratorTable("<Key>Return:set()notify()unset()\n<Key>q:set()notify()unset()");
    accels_cr_click = XtParseAcceleratorTable("<Key>Return:set()notify()unset()\n"
					      "<Btn1Down>:set()\n"
					      "<Btn1Up>:notify()unset()");
#endif

    XtGetApplicationResources(top_level, (XtPointer) & resource,
			      application_resources,
			      XtNumber(application_resources), (ArgList) NULL,
			      0);
    currwin.shrinkfactor = resource.shrinkfactor;
    
    /* Initialize the `debug' resource as early as possible.  Note: earlier
     * calls to TRACE_* or tests for `if (debug)' will only work if the
     * XDVIDEBUG environment variable is set!
     */
    debug |= parse_debugging_option(resource.debug_arg);
    kpathsea_debug = debug / DBG_STAT;

    if (debug)
	fprintf(stderr, "KPATHSEA_DEBUG = %d\n", kpathsea_debug);
    
    kpse_init_prog("XDVI", resource.pixels_per_inch, resource.mfmode, resource.alt_font);
    kpse_set_program_name(argv[0], "xdvi");

    if (debug & DBG_EXPAND) {
	fprintf(stderr, "%s:%d: KPATHSEA variables:\n", __FILE__, __LINE__);
	fprintf(stderr, "%s:%d: SELFAUTOLOC=\"%s\"\n", __FILE__, __LINE__, getenv("SELFAUTOLOC"));
	fprintf(stderr, "%s:%d: SELFAUTODIR=\"%s\"\n", __FILE__, __LINE__, getenv("SELFAUTODIR"));
	fprintf(stderr, "%s:%d: SELFAUTOPARENT=\"%s\"\n", __FILE__, __LINE__, getenv("SELFAUTOPARENT"));
    }
    
    if (resource.regression) {
	/* currently it just turns on everything; what we'd like here is
	   output that's usable for automatic diffs (e.g. independent
	   of window manager state) */
	debug = DBG_ALL;
    }

    /* Check early for whether to pass off to a different xdvi process
     * (-sourceposition argument for reverse source special lookup).
     */
    property_initialize();

    /* error checking and setting of resources according to command line arguments */
    init_check_resources();

    /* use the file selector if neccessary to get dvi_name */
    if (dvi_name == NULL) {
#ifdef SELFILE
#ifdef EXPERIMENTAL_DELAYED_MTKEXPK
	if (!select_filename(dvi_name, False))
	    return 1;
#else
	if (!select_filename(dvi_name, True))
	    return 1;
#endif /* EXPERIMENTAL_DELAYED_MTKEXPK */
#else
	XDVI_ERROR((stderr, "No file name specified."));
	usage(EXIT_FAILURE);
#endif
    }
    ASSERT(dvi_name != NULL, "dvi_name must have been initialized here");
    
    /* if successful, this expands dvi_name to a full path */
    if (!open_dvi_file_wrapper(dvi_name, True, /* from command line? */
			       False, /* launch new xdvi instance? */
			       &tried_dvi_ext)) { 
	return 1;
    }

    
    TRACE_CLIENT((stderr, "dvi_name is: |%s|\n", dvi_name));

    g_dvi_dir_name = get_dir_component(dvi_name);
    ASSERT(g_dvi_dir_name != NULL, "dvi_name must contain a dir component");
    g_dvi_dir_len = strlen(g_dvi_dir_name);

    
    form_dvi_property();

    /*
      If `unique' is active, we might need to pass the file
      to a different instance of xdvi:
    */
    if (resource.unique) {
	Window w1 = 0, w2 = 0;
	if ((w1 = get_xdvi_window_id(True)) != 0 || (w2 = get_xdvi_window_id(False)) != 0) {
	    if (w1 != 0) { /* another xdvi, and same file */
		w2 = w1;
		set_string_property("", atom_raise(), w2);
	    }
	    else { /* another xdvi, and different file */
		set_string_property(dvi_name, atom_newdoc(), w2);
		set_string_property("", atom_raise(), w2);
	    }
	    if (page_arg != NULL) {
		if (strlen(page_arg) == 0) /* special case: treat `+' as last page */
		    set_string_property("+", atom_newpage(), w2);
		else
		    set_string_property(page_arg, atom_newpage(), w2);
		set_string_property("", atom_raise(), w2);
	    }
	    return 0;
	}
	else if (resource.src_fork) {
	    do_fork();
	}
    }

    /*
      Similar for forward search:
     */
    if (resource.src_pos != NULL) {
	Window w;
	if ((w = get_xdvi_window_id(True)) != 0) {
	    /* another instance of xdvi running, displaying the same file */
	    TRACE_CLIENT((stderr, "Match; changing property of client and exiting ..."));
	    set_sourceposition_property(resource.src_pos, w);
	    return 0;
	}
	else if (resource.src_fork) {
	    do_fork();
	}
    }

    /* Needed for source specials and for calling ghostscript. */
    xputenv("DISPLAY", XDisplayString(DISP));


    if (debug) {
	fprintf(stderr, "%s %s, kpathsea %s\n", XDVI_PROGNAME, XDVI_VERSION_INFO, kpathsea_version_string);
	fprintf(stderr,
		"configured with: ppi=%d shrink=%d mfmode=%s alt_font=%s paper=%s\n",
		resource.pixels_per_inch,
		currwin.shrinkfactor,
		resource.mfmode ? resource.mfmode : "<NONE>",
		resource.alt_font,
		resource.paper);
    }
    
#ifdef T1LIB
    /* needed so that T1lib can produce tfm's e.g. for cm-super */
    kpse_set_program_enabled(kpse_tfm_format, True, kpse_src_compile);
#endif
    kpse_set_program_enabled(kpse_any_glyph_format, resource.makepk, kpse_src_compile);
    /* janl 16/11/98: I have changed this. The above line used to
       say the settings in resource.makepk was supplied on the
       commandline, resulting in it overriding _all other_
       settings, derived from the environment or texmf.cnf, no
       matter what the value. The value in resource.makepk could
       be the compile-time default...

       Personaly I like the environment/texmf.cnf to override
       resources and thus changed the 'level' of this setting to
       kpse_src_compile so the environment/texmf.cnf will override
       the values derived by Xt.

       Previous comment here:

       ``Let true values as an X resource/command line override false
       values in texmf.cnf/envvar.''  */

/*     fprintf(stderr, "SELFAUTOPARENT: |%s|\n", kpse_path_expand("$SELFAUTOPARENT")); */
/*     fprintf(stderr, "TEXMFCNF: |%s|\nTEXMFMAIN: |%s|\n", */
/* 	    kpse_path_expand("$TEXMFCNF"), */
/* 	    kpse_path_expand("$TEXMFMAIN")); */

    
    /*
     *		Step 2:  Settle colormap issues.  This should be done before
     *		other widgets are created, so that they get the right
     *		pixel values.  (The top-level widget won't have the right
     *		values, but I don't think that makes any difference.)
     */
    
#ifdef XSERVER_INFO
    print_xserver_info();
#endif

    create_colormaps();

    
#ifdef TESTING_OPEN_FILES
    fprintf(stderr, "open_max: %ld\n", OPEN_MAX);
    for (i = 0; i < OPEN_MAX - 10; i++) {
	FILE *fp;
	if ((fp = fopen("/tmp/foo", "r")) == NULL) {
	    perror("fopen");
	    exit(1);
	}
    }
    fprintf(stderr, "opened %d files.\n", i);
#endif
    

    /* toolbar code may open files, but we have no check close_a_file() in
       the toolbar code; so do this before prescan() possibly opens lots of files.
     */
#ifdef MOTIF
    main_row = XmCreateMainWindow(top_level, "main", NULL, 0);
    
    create_menu_buttons(main_row, &menu_bar);

    /* seems to be needed for enabling `XmNhighlightOnEnter' for the toolbar buttons
       - is this the correct place to do it? */
    XtVaSetValues(top_level, XmNkeyboardFocusPolicy, (XtArgVal)XmPOINTER, NULL);

    tool_bar = create_toolbar(main_row, menu_bar);

    form = XtVaCreateWidget("form", xmFormWidgetClass, main_row,
			    XmNshadowThickness, 0,
			    NULL);
#endif
    
    /*
     *		Step 3:  Initialize the dvi file and set titles.
     */

#ifdef T1LIB
    /*
      At this point DISP, our_visual, our_depth and our_colormap must
      be defined. Also, init_t1() must go before init_dvi_file(), since
      read_postamble will define some fonts and insert them into
      fontmaps_hash, but we need a clean fontmaps_hash for detecting
      duplicate entries in the map file.
    */

    if (resource.t1lib) {
       init_t1();
    }
#endif /* T1LIB */


    setup_signal_handlers();

    /* open and initialize the DVI file, but don't read the fonts yet */
    if (!internal_init_dvi(&errflag,
#ifdef EXPERIMENTAL_DELAYED_MTKEXPK
			   False
#else
			   True
#endif
			   )) {
	if (tried_dvi_ext)
	    XDVI_FATAL((stderr, "Could not open %s: %s, and %s.dvi doesn't exist either - exiting.",
			dvi_name, get_dvi_error(errflag), dvi_name));
	else
	    XDVI_FATAL((stderr, "Could not open %s: %s",
			dvi_name, get_dvi_error(errflag)));
    }
    
    if (page_arg) {
	if (*page_arg == '\0') { /* empty page_arg -> goto last page */
	    current_page = total_pages - 1;
	}
	else {
	    char *testptr;
	    current_page = strtoul(page_arg, &testptr, 10) - 1;
	    if (*testptr != '\0') {
		XDVI_FATAL((stderr, "Invalid page number: `%s'.", page_arg));
	    }
	    current_page = check_goto_page(current_page);
	}
    }

    ASSERT(g_bak_fp != NULL, "Backup file pointer must have been initialized here");
    if (resource.prescan) {
	prescan(g_bak_fp);
    }

    unshrunk_page_w = pageinfo_get_page_width(current_page);
    unshrunk_page_h = pageinfo_get_page_height(current_page);

    init_page();

    /*
     *		Step 4:  Create widgets, and set initial window size.
     */

    /* currently these override expert mode - using this is deprecated
       in favour of `-expertmode'; inform user about this: */
    if (resource.statusline) {
	XDVI_WARNING((stderr, "The option/X resource `statusline' is obsolete; "
		      "use `-expertmode <flag>' instead, e.g. `-expertmode 1'\n"
		      "to switch on the status line, or `-expertmode 6'\n"
		      "to switch it off. See the xdvi man page for details."));
	expert_flag |= XPRT_SHOW_STATUSLINE;
    }

    create_initialize_widgets(
#ifdef MOTIF
			      tool_bar
#endif
			      );

    /* this needs to be done after total_pages is known (via init_dvi_file) */
    get_icon_and_title(dvi_name, &icon_name, &title_name);
    add_icon(top_level, title_name, icon_name);
    /* this needs to be done after the widgets have been created */
    set_icon_and_title(icon_name, title_name);
    free(icon_name);
    free(title_name);
    icon_name = title_name = NULL;

    
    image = XCreateImage(DISP, our_visual, 1, XYBitmap, 0,
			 (char *)NULL, 0, 0, BMBITS, 0);
    image->bitmap_unit = BMBITS;
#ifdef	WORDS_BIGENDIAN
    image->bitmap_bit_order = MSBFirst;
#else
    image->bitmap_bit_order = LSBFirst;
#endif
    {
	short endian = MSBFirst << 8 | LSBFirst;
	image->byte_order = *((char *)&endian);
    }


    /* Store window id for use by src_client_check().  */
    {
#ifndef WORD64
	xuint32 data = XtWindow(top_level);
	XChangeProperty(DISP, DefaultRootWindow(DISP),
			atom_xdvi_windows(), atom_xdvi_windows(), 32,
			PropModePrepend, (unsigned char *)&data, 1);
#else
	unsigned char data[4];
	set_window_id(XtWindow(top_level), data);
	XChangeProperty(DISP, DefaultRootWindow(DISP),
			atom_xdvi_windows(), atom_xdvi_windows(), 32,
			PropModePrepend, data, 1);
#endif
	set_dvi_property();
    }

    /*
     *	Step 5:  Assign colors and GCs.
     *		 Because of the latter, this has to go after the widgets are realized.
     */

    create_gcs();

    create_cursors();

#ifdef EXPERIMENTAL_DELAYED_MTKEXPK
    /* Only now, we initialize the fonts. First update the window: */
    force_statusline_update();
    printlog_create(NULL, NULL, NULL, NULL);
    printlog_popup();
    
    printlog_append_str("\nTrying to create font(s)\n");
    if (!find_postamble(g_bak_fp, &errflag) || !read_postamble(g_bak_fp, &errflag, True)) {
	XDVI_ERROR((stderr, "Couldn't read fonts from postamble: %s\n", get_dvi_error(errflag)));
    }
#endif /* EXPERIMENTAL_DELAYED_MTKEXPK */
    
#ifdef MOTIF
    create_pagelist();
#endif

    do_forward_search(resource.src_pos);

    
#ifdef HTEX
    if (resource.anchor_pos != NULL) {
	g_anchor_pos = xstrdup(resource.anchor_pos);
	ev_flags |= EV_ANCHOR;
    }
#endif /* HTEX */

#if defined(MOTIF) && HAVE_XPM
    tb_check_navigation_sensitivity(current_page);
#endif

#if CHECK_APP_FILEVERSION
    check_app_defaults_fileversion();
#endif

    /* can do this only after scrollbars have been realized */
    if (!BROKEN_RECONFIG && (expert_flag & XPRT_SHOW_SCROLLBARS) == 0) {
	toggle_scrollbars();
    }

    /* initialize file watching */
    watch_file_cb(NULL, NULL);

    do_pages();

    /* NOTREACHED */
    return 0;
}

