/*========================================================================*\

Copyright (c) 1990-2002  Paul Vojta

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

NOTE:
xdvi is based on prior work, as noted in the modification history
in xdvi.c.

\*========================================================================*/

/*
 *	Original version by Eric C. Cooper, CMU
 */

#ifndef	XDVI_H
#define	XDVI_H

#include "xdvi-config.h"

/* headers used by all modules */
#include "xdvi-debug.h"
#include "c-auto.h"

#include "kpathsea/c-auto.h" /* kpathsea definitions */
#include "kpathsea/config.h"

/********************************
 *	The C environment	*
 *******************************/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

/* in case stdlib.h doesn't define these ... */
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#if HAVE_SYS_WAIT_H
# include <sys/wait.h>
#endif
#ifndef WIFEXITED
# define WIFEXITED(status)	(((status) & 255) == 0)
#endif
#ifndef WEXITSTATUS
# define WEXITSTATUS(status)	((unsigned)(status) >> 8)
#endif
#ifndef WIFSIGNALED
# ifndef WIFSTOPPED
#  define WIFSTOPPED(status)	(((status) & 0xff) == 0x7f)
# endif
# define WIFSIGNALED(status)	(!WIFSTOPPED(status) && !WIFEXITED(status))
#endif
#ifndef WTERMSIG
# define WTERMSIG(status)	((status) & 0x7f)
#endif

/*
 * If this is set to 1, xdvi will explicitly clear the internal GS
 * buffer after every page that contained a PS special (figure etc.)
 * to avoid artifacts with later PS specials (overlapping lines etc.);
 * see also bug #633420. This workaround will slow down the drawing of
 * the next page considerably. Currently xdvi also sometimes dies with
 * `Internal error in beginheader_gs()'
 * when paging quickly through a file with this hack enabled.
 * Set to 0 to disable the hack.
 */
#define GS_PIXMAP_CLEARING_HACK 1

#ifdef __hpux
/* On HP-UX 10.10 B and 20.10, compiling with _XOPEN_SOURCE + ..._EXTENDED
 * leads to poll() not realizing that a file descriptor is writable in psgs.c.
 */
# define _HPUX_SOURCE	1
#else
# ifndef _XOPEN_SOURCE
# define _XOPEN_SOURCE	600
# endif
# define _XOPEN_SOURCE_EXTENDED	1
# define __EXTENSIONS__	1	/* needed to get struct timeval on SunOS 5.5 */
# define _SVID_SOURCE	1	/* needed to get S_IFLNK in glibc */
# define _BSD_SOURCE	1	/* needed to get F_SETOWN in glibc-2.1.3 */
#endif

/* #ifndef MOTIF */
/* #define XAW 1 */
/* #endif */

/* Some O/S dependent kludges. */
#ifdef _AIX
# define _ALL_SOURCE 1
#endif

/* just a kludge, no real portability here ... */
#define DIR_SEPARATOR '/'

#if STDC_HEADERS
# include <stddef.h>
# include <stdlib.h>
/* the following works around the wchar_t problem */
# include <X11/X.h>
# if HAVE_X11_XOSDEFS_H
#  include <X11/Xosdefs.h>
# endif
# ifdef X_NOT_STDC_ENV
#  undef X_NOT_STDC_ENV
#  undef X_WCHAR
#  include <X11/Xlib.h>
#  define X_NOT_STDC_ENV
# endif
#endif

/* For wchar_t et al., that the X files might want. */
#include "kpathsea/systypes.h"
#include "kpathsea/c-memstr.h"

#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h> /* this should define MAXPATHLEN */
#endif

#include <X11/Xlib.h>	/* include Xfuncs.h, if available */
#include <X11/Xutil.h>	/* needed for XDestroyImage */
#include <X11/Xos.h>

#include <X11/Xfuncs.h>
#include <X11/Intrinsic.h>

#ifndef MAXPATHLEN
# ifdef PATH_MAX
#  define MAXPATHLEN PATH_MAX
# else
#  define MAXPATHLEN 1024
# endif
#endif

#if defined(CFG2RES) && !defined(SELFAUTO)
# define SELFAUTO 1
#endif

#if defined(SELFAUTO) && !defined(DEFAULT_CONFIG_PATH)
# define DEFAULT_CONFIG_PATH "$SELFAUTODIR:$SELFAUTOPARENT"
#endif

/* NOTE: we don't use CFGFILE */

typedef	char		Bool3;		/* Yes/No/Maybe */

#define	True	1
#define	False	0
#define	Maybe	2


#ifdef DEBUG
#include<asm/msr.h>
extern unsigned long time_start, time_end;
#endif


#ifndef WORD64
# ifdef LONG64
typedef unsigned int xuint32;
# else
typedef unsigned long xuint32;
# endif
#endif

/* for unused parameters */
#ifndef UNUSED
#define UNUSED(x) ((void)(x))
#endif

#ifdef	VMS
#include <string.h>
#define	index	strchr
#define	rindex	strrchr
#define	bzero(a, b)	(void) memset ((void *) (a), 0, (size_t) (b))
#define bcopy(a, b, c)  (void) memmove ((void *) (b), (void *) (a), (size_t) (c))
#endif

#include <stdio.h>
#include <setjmp.h>

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

/* all of these are POSIX and should have been defined by unistd.h: */
#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif
#ifndef STDOUT_FILENO
#define STDOUT_FILENO 1
#endif
#ifndef STDERR_FILENO
#define STDERR_FILENO 2
#endif

/* see C FAQ; additional +1 for '\0' */
#define LENGTH_OF_INT ((sizeof(int) * CHAR_BIT + 2) / 3 + 1 + 1)
#define LENGTH_OF_LONG ((sizeof(long) * CHAR_BIT + 2) / 3 + 1 + 1)
#define LENGTH_OF_ULONG ((sizeof(unsigned long) * CHAR_BIT + 2) / 3 + 1 + 1)

#include "kpathsea/c-dir.h" /* dirent.h, NAMLEN */

extern KPSEDLL char *kpathsea_version_string;

#ifndef	NeedFunctionPrototypes
# if	__STDC__
#  define   NeedFunctionPrototypes	1
# else
#  define   NeedFunctionPrototypes	0
# endif
#endif

#ifndef	NeedWidePrototypes
# define NeedWidePrototypes	NeedFunctionPrototypes
#endif

#ifndef	NeedVarargsPrototypes
# define NeedVarargsPrototypes	NeedFunctionPrototypes
#endif

#include "kpathsea/c-vararg.h"

#ifndef	_XFUNCPROTOBEGIN
# define _XFUNCPROTOBEGIN
# define _XFUNCPROTOEND
#endif

#ifndef	_Xconst
# if __STDC__
#  define	_Xconst	const
# else
#  define	_Xconst /* as nothing */
# endif
#endif

/* If xmkmf is broken and there's a symlink from /usr/include/X11 to the right
 * place, then there will be no -I... argument on the cc command line for the
 * X include files.  Since gcc version 3 and higher sets __STDC__ to 0 when
 * including system header files on some platforms, we may end up with
 * NeedFunctionPrototypes set to 0 when it should be 1.  So, let's force the
 * issue.
 */
#if __STDC__ && !defined(FUNCPROTO)
/* FUNCPROTO is a bitmask specifying ANSI conformance (see Xfuncproto.h).
   The single bits specify varargs, const availability, prototypes etc.;
   we enable everything here. */
# define FUNCPROTO (-1)
#endif

#ifndef	VOLATILE
# if __STDC__ || (defined(__stdc__) && defined(__convex__))
#  define VOLATILE	volatile
# else
#  define VOLATILE	/* as nothing */
# endif
#endif

#ifndef	NORETURN
# ifdef	__GNUC__
#  ifndef __STRICT_ANSI__
#   define	NORETURN	volatile
#  else
#   define	NORETURN	/* as nothing */
#  endif
# else
#  define	NORETURN	/* as nothing */
# endif
#endif

#ifndef	OPEN_MODE
/*
 * SU, 2001/01/07: xdvi defines OPEN_MODE as "r" or as "r", "ctx=stm" (for VMS),
 * but we use the definition of FOPEN_R_MODE from kpathsea/c-fopen.h instead:
 */
# define OPEN_MODE FOPEN_R_MODE
#endif	/* OPEN_MODE */

#ifndef	VMS
# define OPEN_MODE_ARGS	_Xconst char *
#else
# define OPEN_MODE_ARGS	_Xconst char *, _Xconst char *
#endif

#ifndef __LINE__
# define __LINE__ 0
#endif

#ifndef __FILE__
# define __FILE__ "?"
#endif

#define	MAXDIM 32767

typedef	unsigned char	ubyte;

#if NeedWidePrototypes
typedef	unsigned int	wide_ubyte;
typedef	int		wide_bool;
# define WIDENINT	(int)
#else
typedef	ubyte		wide_ubyte;
typedef	Boolean		wide_bool;
# define WIDENINT
#endif

#if defined(MAKEPK) && !defined(MKTEXPK)
# define MKTEXPK 1
#endif

#define	spell_conv0(n, f)   ((long) (n * f))
#define	spell_conv(n)	    spell_conv0(n, dimconv)

typedef BMTYPE bm_type_t;
typedef unsigned BMTYPE bm_unit_t;
/* #define	BMUNIT		    unsigned BMTYPE */
#define	BMBITS		    (8 * BMBYTES) /* number of bits in a bm_type_t */

#define	ADD(a, b)	((bm_unit_t *) (((char *) a) + b))
#define	SUB(a, b)	((bm_unit_t *) (((char *) a) - b))

extern bm_unit_t bit_masks[BMBITS + 1];

/* for safely printing char *s that might be NULL */
#define STRING_OR_NULL(x) ((x == NULL ? "<NULL>" : x))

#define INSIDE_MANE_WIN ((currwin.win == mane.win))
#define MAGNIFIER_ACTIVE ((magnifier.win != 0))

struct frame {
    /* dvi_h and dvi_v is the horizontal and vertical baseline position;
       it is the responsability of the set_char procedure to update
       them. */
    struct framedata {
	long dvi_h, dvi_v, w, x, y, z;
	int pxl_v;
    } data;
    struct frame *next, *prev;
};

#ifdef TEXXET
typedef void setchar_retval_t;
#else
typedef long setchar_retval_t;
#endif

typedef	setchar_retval_t (*set_char_proc) (
#ifdef TEXXET
					   wide_ubyte cmd,
#endif
					   wide_ubyte ch);

#define ROUNDUP(x,y) (((x)+(y)-1)/(y))

#ifndef	BDPI
#define	BDPI	600
#endif

#if defined(GS_PATH) && !defined(PS_GS)
# define PS_GS	1
#endif

#if defined(PS_DPS) || defined(PS_NEWS) || defined(PS_GS)
# define PS	1
#else
# define PS	0
#endif


/* A list of currently covered encodings; a mapping
   of these to symbolic names is defined in xdvi.c */
typedef enum {
    ISO_8859_1,
    UTF_8,
    INVALID_ENC
} encoding_t;

typedef enum {
    XPRT_SHOW_NONE = 0U,
    XPRT_SHOW_STATUSLINE = 1,
    XPRT_SHOW_SCROLLBARS = 2,
#ifdef MOTIF
    XPRT_SHOW_PAGELIST = 4,
    XPRT_SHOW_TOOLBAR = 8,
    XPRT_SHOW_MENUBAR = 16,
    XPRT_SHOW_ALL = 31
#else
    XPRT_SHOW_BUTTONS = 4,
    XPRT_SHOW_ALL = 7
#endif
} expert_flag_t;

extern expert_flag_t expert_flag;

/*
 *	X resources.
 */
extern	struct _resource {
    /* NOTE: we don't use CFGFILE */
    Boolean	    regression;
    int		    app_defaults_fileversion;
    Boolean	    ruler_mode;
    Boolean	    use_tex_pages;
    int		    shrinkfactor;
    _Xconst char    *main_translations;
    _Xconst char    *wheel_translations;
    int		    wheel_unit;
    int		    density;
#ifdef	GREY
    float	    gamma;
    float	    inverted_factor;
#endif
    int		    pixels_per_inch;
    Boolean	    delay_rulers;
    int		    tick_length;
    char	    *tick_units;
    _Xconst char    *sidemargin;
    _Xconst char    *topmargin;
    _Xconst char    *xoffset;
    _Xconst char    *yoffset;
    _Xconst char    *paper;
    Boolean         paper_landscape; /* only used internally */
    _Xconst char    *alt_font;
#ifdef MKTEXPK
    Boolean	    makepk;
#endif
    _Xconst char    *mfmode;
    _Xconst char    *editor;
    Boolean	    t1lib;
    const char      *src_pos;
    const char      *text_encoding;
    encoding_t      internal_encoding;
    Boolean	    src_fork;
    _Xconst char    *sub_pixels;
    Boolean	    unique;
    Boolean	    list_fonts;
    Boolean	    reverse;
    Boolean	    warn_spec;
    Boolean	    hush_chars;
    Boolean	    hush_chk;
    Boolean	    hush_stdout;
    Boolean	    safer;
#ifdef VMS
    _Xconst char    *fore_color;
    _Xconst char    *back_color;
#endif
    Pixel	    fore_Pixel;
    Pixel	    back_Pixel;
    Pixel	    brdr_Pixel;
    Pixel	    hl_Pixel;
    Pixel	    cr_Pixel;
    _Xconst char    *icon_geometry;
    Boolean	    keep_flag;
    Boolean	    copy;
    Boolean	    thorough;
    Boolean	    pause;
    _Xconst char    *pause_special;
    Boolean	    fullscreen;
#ifdef PS
    int		    postscript;
    Boolean	    allow_shell;
# ifdef	PS_DPS
    Boolean	    useDPS;
# endif
# ifdef	PS_NEWS
    Boolean	    useNeWS;
# endif
# ifdef	PS_GS
    Boolean	    useGS;
    Boolean	    gs_safer;
    Boolean	    gs_alpha;
    _Xconst char    *gs_path;
    _Xconst char    *gs_palette;
# endif
# ifdef	MAGICK
    Boolean	    useMAGICK;
    _Xconst char    *magick_cache;
# endif
#endif	/* PS */
    Boolean         prescan;
    Boolean         use_temp_fp;
    _Xconst char    *debug_arg;
    Boolean	    version_flag;
    _Xconst char    *menu_translations;
    Boolean	    expert;
    float	    watch_file; /* check DVI file every so often, in seconds */
    int             expert_mode;
#ifndef MOTIF
    Dimension	    btn_side_spacing;
    Dimension	    btn_top_spacing;
    Dimension	    btn_between_spacing;
    /* ignored, only for backwards compatibility */
    Dimension	    btn_between_extra;
    Dimension	    btn_border_width;
#else /* MOTIF */
    _Xconst char    *toolbar_translations;
#endif /* MOTIF */
    Boolean	    statusline;
#ifdef MOTIF
    /* not a user-level resource; indicates a problem with the toolbar, in
       which case the toolbar is disabled. */
    Boolean	    toolbar_unusable;
    char	    *toolbar_pixmap_file;
    Boolean	    toolbar_buttons_raised;
#endif
    Boolean	    pagelist_highlight_current;
    Dimension	    pagelist_width;
    _Xconst char    *mg_arg[5];
#if COLOR
    Boolean         use_color;
#endif
#ifdef GREY
    Boolean	    use_grey;
    Bool3	    install;
#endif
    _Xconst char    *dvips_path;
    _Xconst char    *ps2pdf_path;
    int		    dvips_hang;
    int		    dvips_fail_hang;
    _Xconst char    *dvips_printer_str;
    _Xconst char    *dvips_options_str;
    char	    *rule_color;
    Pixel	    rule_pixel;
#ifdef HTEX
    int		    link_style;
    char	    *link_color;
    char	    *visited_link_color;
    char	    *browser;
    char	    *CMD_WWWbrowser;
    char	    *unknown_mime_suffix;
    char	    *no_mime_suffix;
    char	    *anchor_pos;
    /*     char	    *_scroll_pages; */
#endif /* HTEX */
    char	    *help_general;
#ifdef HTEX
    char	    *help_hypertex;
#endif
    char	    *help_othercommands;
    char	    *help_pagemotion;
    char	    *help_marking;
    char	    *help_rulermode;
    char	    *help_search;
    char	    *help_mousebuttons;
    char	    *help_sourcespecials;
} resource;


/*
 * This was MOTIF_TIMERS in the non-k xdvi version, but since xdvik
 * also uses XtAppAddTimeOut() for Xaw, we always need to define it.
 * See the comment in events.c for further explanations.
 */
#define TIMER_HACK 1

/* globally used GUI stuff */
#ifdef MOTIF
#include <Xm/Xm.h>
/* we use TIMER_HACK instead (see above) */
/* # ifndef MOTIF_TIMERS */
/* #  define MOTIF_TIMERS (XmVERSION >= 2) */
/* # endif */
extern  XmStringCharSet g_charset;

# ifndef DDIST
#  define DDIST 4
# endif
# ifndef DDIST_MAJOR
#  define DDIST_MAJOR 10
# endif
# ifndef DDIST_MINOR
#  define DDIST_MINOR 5
# endif
#endif
extern XtAppContext app;


extern unsigned int page_w, page_h;
extern int		offset_x, offset_y;
extern unsigned int	unshrunk_paper_w, unshrunk_paper_h;
extern unsigned int	unshrunk_page_w, unshrunk_page_h;

extern char *dvi_name;	/* dvi file name */
extern char *g_dvi_dir_name;
extern size_t g_dvi_dir_len;

extern	FILE		*g_bak_fp; /* temporary backup file, or original file if `-notempfile' is used */

extern	time_t		dvi_time;		/* last modification time */
extern	unsigned char	*dvi_property;		/* for setting in window */
extern	size_t		dvi_property_length;
extern	_Xconst char	*prog;
extern	int		bak_shrink;		/* last shrink factor != 1 */
extern	Dimension	window_w, window_h;
extern	XImage		*image;
extern	int		backing_store;
extern	int		home_x, home_y;

extern	Display		*DISP;
extern	Screen		*SCRN;

extern	XtAccelerators	accels_cr, accels_cr_click;
#ifdef GREY
extern	int		screen_number;
extern	Visual		*our_visual;
extern	unsigned int	our_depth;
extern	Colormap	our_colormap;
#else
# define our_depth	(unsigned int) DefaultDepthOfScreen(SCRN)
# define our_visual	DefaultVisualOfScreen(SCRN)
# define our_colormap	DefaultColormapOfScreen(SCRN)
#endif

extern	GC	ruleGC;
extern	GC	foreGC;
extern	GC	invertedGC;
extern  GC	highGC;
extern  GC	linkcolorGC;
extern  GC	visited_linkcolorGC;
extern	GC	foreGC2;
extern	GC	copyGC;
extern  GC	rulerGC;
extern	Boolean	copy;

extern	Cursor	wait_cursor;
extern	Cursor	ready_cursor;
extern	Cursor	corrupted_cursor;
extern	Cursor	link_cursor;
extern	Cursor  rule_cursor;
extern	Cursor  mag_cursor;
extern	Cursor	drag_cursor_h;
extern	Cursor	drag_cursor_v;
extern	Cursor	drag_cursor_a;
extern	Cursor	pause_cursor;

extern unsigned long cursor_flags;
#define LINK_CURSOR 1
#define MAG_CURSOR 2
/* flags for drag cursors */
#define DRAG_VERTICAL 4
#define DRAG_HORIZONTAL 8
#define DRAG_ALL 16
#define CORRUPTED_CURSOR 32	/* if file is corrupted */

struct	WindowRec {
    Window win;
    int	shrinkfactor;
    int	base_x;
    int	base_y;
    unsigned int width;
    unsigned int height;
    /* for pending expose events */
    int	min_x;
    int max_x;
    int min_y;
    int max_y;
};

extern struct WindowRec mane;
extern struct WindowRec currwin;

extern Widget top_level;
#ifndef MOTIF
extern Widget vport_widget;
extern Widget form_widget;
extern Widget paned;
#else
extern Widget main_window, main_row, tool_bar, top_row, form, menu_bar;
#endif /* MOTIF */

extern Widget x_bar, y_bar;	/* horizontal and vert. scroll bars */
extern Widget draw_widget, draw_background, clip_widget;

#ifdef STATUSLINE
/* this is only for the initialization; the statusline will reset it to a more adequate value: */
# define XTRA_H	17
#endif

/*
 * Generic structure for DVI scans; contains a buffer for longjmp()ing
 * out of the scanning process if it's interrupted by the user (ugh ...)
 * and a generic `void *' to scan-specific info.
 */
struct scan_info_t {
    jmp_buf done_env;
    void (*geom_special)(struct scan_info_t *, _Xconst char *);
    void *data;
};

/*
 * Used by the geometry-scanning routines.
 * It passes pointers to routines to be called at certain
 * points in the dvi file, and other information.
 */
struct geom_info_t {
    void (*geom_box)(struct scan_info_t *, long, long, long, long);
    void *geom_data;
};

#ifdef	CFG2RES
struct cfg2res {
    _Xconst char    *cfgname;	/* name in config file */
    _Xconst char    *resname;	/* name of resource */
    Boolean	    numeric;	/* if numeric */
};
#endif

#define one(fp)		((unsigned char) getc(fp))
#define sone(fp)	((long) one(fp))
#define two(fp)		num(fp, 2)
#define stwo(fp)	snum(fp, 2)
#define four(fp)	num(fp, 4)
#define sfour(fp)	snum(fp, 4)

extern void get_icon_and_title(char *filename, char **icon_name, char **title_name);
extern void set_icon_and_title(const char *icon_name, const char *title_name);
extern void reconfigure_window(Boolean fullsize, Dimension w, Dimension h,
			       Boolean save_position);
extern void set_windowsize(Dimension *ret_w, Dimension *ret_h, int add_w, int add_h, Boolean override);

#ifdef MOTIF
extern void motif_translations_kludge(void);
#endif

#ifndef	MAX
# define MAX(i, j)       ( (i) > (j) ? (i) : (j) )
#endif

#define REPORT_XDVI_BUG_TEMPLATE "Please report this as a bug to:\n\
   http://sourceforge.net/tracker/?group_id=23164&atid=377580\n"


#endif	/* XDVI_H */
